package sra.jun.goodies.animation;

import java.awt.Color;
import java.awt.Component;
import java.awt.Container;
import java.awt.Graphics;
import java.awt.Label;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.Window;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import sra.smalltalk.DependentEvent;
import sra.smalltalk.DependentListener;
import sra.smalltalk.StComposedText;
import sra.smalltalk.StImage;
import sra.smalltalk.StRectangle;
import sra.smalltalk.StView;
import sra.smalltalk.StViewCanvas;
import sra.smalltalk.StViewPanel;
import sra.smalltalk.WindowSpecLayout;
import sra.jun.geometry.basic.Jun2dPoint;
import sra.jun.goodies.button.JunButtonViewAwt;
import sra.jun.goodies.images.support.JunImageAdjuster;
import sra.jun.goodies.track.JunTrackSliderViewAwt;

/**
 * JunCartoonMovieViewAwt class
 * 
 *  @author    Nobuto Matsubara
 *  @created   2004/02/05 (By Nobuto Matsubara)
 *  @updated   N/A
 *  @version   445 (with StPL6.5) based on Jun519 for Smalltalk
 *  @copyright 1999-2004 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2004 Information-technology Promotion Agency, Japan (IPA)
 *  @copyright 2001-2004 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunCartoonMovieViewAwt.java,v 6.10 2004/04/15 06:27:53 nisinaka Exp $
 */
public class JunCartoonMovieViewAwt extends StViewCanvas implements JunCartoonMovieView {

	protected StImage currentImage;
	protected StImage adjustedImage;
	protected Color backgroundColor;

	protected JunButtonViewAwt firstButtonView;
	protected JunButtonViewAwt lastButtonView;
	protected JunButtonViewAwt firstMarkerButtonView;
	protected JunButtonViewAwt lastMarkerButtonView;
	protected JunButtonViewAwt loopButtonView;
	protected JunButtonViewAwt nextButtonView;
	protected JunButtonViewAwt playButtonView;
	protected JunButtonViewAwt previousButtonView;
	protected JunTrackSliderViewAwt sliderView;
	protected JunTrackSliderViewAwt sliderView2;
	protected Label progressTextView;

	/**
	 * Create instance and initialize.
	 * 
	 * @param aJunCartoonMovie sra.jun.goodies.animation.JunCartoonMovie
	 * @category Instance creattion
	 */
	public JunCartoonMovieViewAwt(JunCartoonMovie aJunCartoonMovie) {
		super(aJunCartoonMovie);
	}

	/**
	 * Initializing.
	 * 
	 * @category initialize-release
	 */
	public void initialize() {
		super.initialize();
		currentImage = null;
		adjustedImage = null;
		backgroundColor = null;
	}

	/**
	 * Displaying an animation.
	 * 
	 * @param aGraphics java.awt.Graphics
	 * @category displaying
	 */
	public void displayOn_(Graphics aGraphics) {
		if (this.getCartoonMovie().isEmpty()) {
			return;
		}
		StImage anImage = this.getCartoonMovie().imageToDisplay();
		if (anImage.equals(currentImage)) {
			// this.clearAreasOutsideOn_(aGraphics);
			Rectangle viewBounds = this.getBounds();
			Rectangle imageBounds = adjustedImage.bounds();
			int x = (viewBounds.x + viewBounds.width / 2) - (imageBounds.x + imageBounds.width / 2);
			int y = (viewBounds.y + viewBounds.height / 2) - (imageBounds.y + imageBounds.height / 2);
			adjustedImage.displayOn_at_(aGraphics, new Point(x, y));
		} else {
			if (anImage.bounds().width > this.getBounds().width || anImage.bounds().height > this.getBounds().height) {
				adjustedImage = JunImageAdjuster.Adjust_extent_(anImage, this.getSize());
			} else {
				adjustedImage = anImage;
			}
			StRectangle aRectangle = new StRectangle(adjustedImage.bounds());
			aRectangle = aRectangle.align_with_(aRectangle.center(), (new StRectangle(this.getBounds())).center());
			StRectangle[] rectangles = (new StRectangle(this.getBounds())).areasOutside_(aRectangle);
			for (int i = 0; i < rectangles.length; ++i) {
				StRectangle rect = rectangles[i];
				aGraphics.setColor(this.backgroundColor());
				aGraphics.fillRect(rect.x(), rect.y(), rect.width(), rect.height());
			}
			Rectangle viewBounds = this.getBounds();
			Rectangle imageBounds = adjustedImage.bounds();
			int x = (viewBounds.x + viewBounds.width / 2) - (imageBounds.x + imageBounds.width / 2);
			int y = (viewBounds.y + viewBounds.height / 2) - (imageBounds.y + imageBounds.height / 2);
			adjustedImage.displayOn_at_(aGraphics, new Point(x, y));
			currentImage = anImage;
		}
		String aString = this.getCartoonMovie().displayName();
		if (aString == null) {
			return;
		}
		StComposedText aComposedText = new StComposedText(aString);
		StRectangle aBox = new StRectangle(aComposedText.bounds());
		StRectangle aRectangle = (new StRectangle(this.getBounds()).insetBy_(1));
		Jun2dPoint aPoint = new Jun2dPoint((aBox.align_with_(aBox.bottomLeft(), aRectangle.bottomLeft())).origin());
		Jun2dPoint[] points = this.neighborhoodPoints_(aPoint);
		for (int i = 0; i < points.length; ++i) {
			Point p = points[i]._toPoint();
			aGraphics.setColor(Color.white);
			aComposedText.displayOn_at_(aGraphics, p);
		}
		aGraphics.setColor(this.getCartoonMovie().defaultDisplayNameColor());
		aComposedText.displayOn_at_(aGraphics, aPoint._toPoint());
	}

	/**
	 * Clear areas outside.
	 * 
	 * @param graphicsContext Grahics
	 * @category displaying
	 */
	public void clearAreasOutsideOn_(Graphics graphicsContext) {
		StRectangle aRectangle = new StRectangle(adjustedImage.bounds());
		StRectangle thisBounds = new StRectangle(this.getBounds());
		aRectangle = aRectangle.align_with_(aRectangle.center(), thisBounds.center());
		StRectangle[] areas = thisBounds.areasOutside_(aRectangle);
		for (int i = 0; i < areas.length; i++) {
			StRectangle area = areas[i];
			graphicsContext.setColor(this.backgroundColor());
			graphicsContext.fillRect(area.x(), area.y(), area.width(), area.height());
		}
	}

	/**
	 * Update a cartoon view.
	 * 
	 * @param evt sra.smalltalk.DependentEvent
	 * @category updating
	 */
	public void update_(DependentEvent evt) {
		if (!this.isShowing()) {
			return;
		}
		if (this.getCartoonMovie().isEmpty()) {
			return;
		}
		if (evt.getAspect() == $("image")) {
			this.displayOn_(this.getGraphics());
			return;
		}
		if (evt.getAspect() == $("progressText")) {
			this.progressTextView().setText((String) this.getCartoonMovie().progressText().value());
			return;
		}
		super.update_(evt);
	}

	/**
	 * Answer background color.
	 * 
	 * @return java.awt.Color
	 * @category visual properties
	 */
	public Color backgroundColor() {
		if (backgroundColor == null) {
			backgroundColor = Color.white;
		}
		return backgroundColor;
	}

	/**
	 * Set background color.
	 * 
	 * @param colorValue java.awt.Color
	 * @category  visual properties
	 */
	public void backgroundColor_(Color colorValue) {
		backgroundColor = colorValue;
	}

	/**
	 * Answer neighborhoodPoints.
	 * 
	 * @param  aPoint Jun2dPoint
	 * @return Jun2dPoint[]
	 * @category private
	 */
	private Jun2dPoint[] neighborhoodPoints_(Jun2dPoint aPoint) {
		Jun2dPoint[] array = new Jun2dPoint[8];
		array[0] = aPoint.plus_(new Jun2dPoint(1, 0));
		array[1] = aPoint.plus_(new Jun2dPoint(1, 1));
		array[2] = aPoint.plus_(new Jun2dPoint(0, 1));
		array[3] = aPoint.plus_(new Jun2dPoint(-1, 1));
		array[4] = aPoint.plus_(new Jun2dPoint(-1, 0));
		array[5] = aPoint.plus_(new Jun2dPoint(-1, -1));
		array[6] = aPoint.plus_(new Jun2dPoint(0, -1));
		array[7] = aPoint.plus_(new Jun2dPoint(1, -1));
		return array;
	}

	/**
	 * Answer a JunCartoonMovie.
	 * 
	 * @return sra.jun.goodies.animation.JunCartoonMovie
	 * @category accessing
	 */
	public JunCartoonMovie getCartoonMovie() {
		return (JunCartoonMovie) this.model();
	}

	/**
	 * Set bounds.
	 * 
	 * @param The new x-coordinate of this component.
	 * @param The new y-coordinate of this component.
	 * @param The new width-coordinate of this component.
	 * @param The new height-coordinate of this component.
	 * @category bounds accessing
	 */
	public void setBounds(int x, int y, int width, int height) {
		super.setBounds(x, y, width, height);
		currentImage = null;
		adjustedImage = null;
	}

	/**
	 * Build this component.
	 * 
	 * @category Interface-opening
	 */
	public static StView WithTracker(JunCartoonMovie aCartoonMovie) {
		return new StViewPanel(aCartoonMovie) {

			protected JunCartoonMovieViewAwt cartoonView;

			protected JunCartoonMovieViewAwt cartoonView() {
				if (cartoonView == null) {
					cartoonView = new JunCartoonMovieViewAwt((JunCartoonMovie) this.model());
				}
				return cartoonView;
			}

			protected void buildComponent() {
				this.setLayout(new WindowSpecLayout());
				this.add(this.cartoonView(), WindowSpecLayout.Position(0f, 1, 0f, 1, 1f, -1, 1f, -58));
				this.add(this.cartoonView().progressTextView(), WindowSpecLayout.Position(0f, 1, 1f, -57, 1f, -1, 1f, -39));
				this.add(this.cartoonView().firstButtonView(), WindowSpecLayout.Position(0f, 1, 1f, -19, 0f, 19, 1f, -1));
				this.add(this.cartoonView().lastButtonView(), WindowSpecLayout.Position(0f, 20, 1f, -19, 0f, 38, 1f, -1));
				this.add(this.cartoonView().previousButtonView(), WindowSpecLayout.Position(0f, 39, 1f, -19, 0f, 57, 1f, -1));
				this.add(this.cartoonView().nextButtonView(), WindowSpecLayout.Position(0f, 58, 1f, -19, 0f, 76, 1f, -1));
				this.add(this.cartoonView().sliderView(), WindowSpecLayout.Position(0f, 77, 1f, -19, 1f, -77, 1f, -1));
				this.add(this.cartoonView().sliderView2(), WindowSpecLayout.Position(0f, 1, 1f, -38, 1f, -1, 1f, -20));
				this.add(this.cartoonView().loopButtonView(), WindowSpecLayout.Position(1f, -76, 1f, -19, 1f, -58, 1f, -1));
				this.add(this.cartoonView().playButtonView(), WindowSpecLayout.Position(1f, -57, 1f, -19, 1f, -39, 1f, -1));
				this.add(this.cartoonView().firstMarkerButtonView(), WindowSpecLayout.Position(1f, -38, 1f, -19, 1f, -20, 1f, -1));
				this.add(this.cartoonView().lastMarkerButtonView(), WindowSpecLayout.Position(1f, -19, 1f, -19, 1f, -1, 1f, -1));
				this.setSize(300, 300);
			}

			public void _setupKeyboard(Window aWindow) {
				this.cartoonView()._setupKeyboard(aWindow);
			}
		};
	}

	/**
	 * Answer my first button view.
	 *
	 * @return sra.jun.goodies.button.JunButtonViewAwt
	 * @category interface-opening
	 */
	public JunButtonViewAwt firstButtonView() {
		if (firstButtonView == null) {
			firstButtonView = new JunButtonViewAwt(this.getCartoonMovie().trackerModel().firstButton());
		}
		return firstButtonView;
	}

	/**
	 * Answer my last button view.
	 *
	 * @return sra.jun.goodies.button.JunButtonViewAwt
	 * @category interface-opening
	 */
	public JunButtonViewAwt lastButtonView() {
		if (lastButtonView == null) {
			lastButtonView = new JunButtonViewAwt(this.getCartoonMovie().trackerModel().lastButton());
		}
		return lastButtonView;
	}

	/**
	 * Answer my firstMarker button view.
	 *
	 * @return sra.jun.goodies.button.JunButtonViewAwt
	 * @category interface-opening
	 */
	public JunButtonViewAwt firstMarkerButtonView() {
		if (firstMarkerButtonView == null) {
			firstMarkerButtonView = new JunButtonViewAwt(this.getCartoonMovie().trackerModel().firstMarkerButton());
		}
		return firstMarkerButtonView;
	}

	/**
	 * Answer my lastMarker button view.
	 *
	 * @return sra.jun.goodies.button.JunButtonViewAwt
	 * @category interface-opening
	 */
	public JunButtonViewAwt lastMarkerButtonView() {
		if (lastMarkerButtonView == null) {
			lastMarkerButtonView = new JunButtonViewAwt(this.getCartoonMovie().trackerModel().lastMarkerButton());
		}
		return lastMarkerButtonView;
	}

	/**
	 * Answer my loop button view.
	 *
	 * @return sra.jun.goodies.button.JunButtonViewAwt
	 * @category interface-opening
	 */
	public JunButtonViewAwt loopButtonView() {
		if (loopButtonView == null) {
			loopButtonView = new JunButtonViewAwt(this.getCartoonMovie().trackerModel().loopButton());
		}
		return loopButtonView;
	}

	/**
	 * Answer my previous button view.
	 *
	 * @return sra.jun.goodies.button.JunButtonViewAwt
	 * @category interface-opening
	 */
	public JunButtonViewAwt previousButtonView() {
		if (previousButtonView == null) {
			previousButtonView = new JunButtonViewAwt(this.getCartoonMovie().trackerModel().previousButton());
		}
		return previousButtonView;
	}

	/**
	 * Answer my next button view.
	 *
	 * @return sra.jun.goodies.button.JunButtonViewAwt
	 * @category interface-opening
	 */
	public JunButtonViewAwt nextButtonView() {
		if (nextButtonView == null) {
			nextButtonView = new JunButtonViewAwt(this.getCartoonMovie().trackerModel().nextButton());
		}
		return nextButtonView;
	}

	/**
	 * Answer my play button view.
	 *
	 * @return sra.jun.goodies.button.JunButtonViewAwt
	 * @category interface-opening
	 */
	public JunButtonViewAwt playButtonView() {
		if (playButtonView == null) {
			playButtonView = new JunButtonViewAwt(this.getCartoonMovie().trackerModel().playButton());
		}
		return playButtonView;
	}

	/**
	 * Answer my first slider view.
	 *
	 * @return sra.jun.goodies.track.JunTrackSliderViewAwt
	 * @category interface-opening
	 */
	public JunTrackSliderViewAwt sliderView() {
		if (sliderView == null) {
			sliderView = new JunTrackSliderViewAwt(this.getCartoonMovie().trackerModel());
		}
		return sliderView;
	}

	/**
	 * Answer my second slider view.
	 *
	 * @return sra.jun.goodies.track.JunTrackSliderViewAwt
	 * @category interface-opening
	 */
	public JunTrackSliderViewAwt sliderView2() {
		if (sliderView2 == null) {
			sliderView2 = new JunTrackSliderViewAwt(this.getCartoonMovie().trackerModel2());
		}
		return sliderView2;
	}

	/**
	 * Answer my progressText view.
	 * 
	 * @return sra.jun.goodies.JunDisplayViewAwt
	 * @category interface-opening
	 */
	public Label progressTextView() {
		if (progressTextView == null) {
			progressTextView = new Label((String) this.getCartoonMovie().progressText().value());
			progressTextView.setAlignment(Label.CENTER);
			progressTextView.setBackground(new Color(0.812477f, 1.0f, 0.624954f));
			this.getCartoonMovie().progressText().addDependentListener(new DependentListener() {
				public void update_(DependentEvent evt) {
					progressTextView.setText((String) getCartoonMovie().progressText().value());
				}
			});
		}
		return progressTextView;
	}

	/**
	 * Set up the keyboard for the view on the window.
	 * 
	 * @param aWindow java.awt.Window
	 * @see sra.smalltalk.StView#_setupKeyboard(java.awt.Window)
	 * @category keyboard
	 */
	public void _setupKeyboard(final Window aWindow) {
		this.addKeyListener(this.getCartoonMovie()._keyListener());
		this.progressTextView().addKeyListener(this.getCartoonMovie()._keyListener());
		this.firstButtonView().addKeyListener(this.getCartoonMovie()._keyListener());
		this.lastButtonView().addKeyListener(this.getCartoonMovie()._keyListener());
		this.previousButtonView().addKeyListener(this.getCartoonMovie()._keyListener());
		this.nextButtonView().addKeyListener(this.getCartoonMovie()._keyListener());
		this.sliderView().addKeyListener(this.getCartoonMovie()._keyListener());
		this.sliderView2().addKeyListener(this.getCartoonMovie()._keyListener());
		this.loopButtonView().addKeyListener(this.getCartoonMovie()._keyListener());
		this.playButtonView().addKeyListener(this.getCartoonMovie()._keyListener());
		this.firstMarkerButtonView().addKeyListener(this.getCartoonMovie()._keyListener());
		this.lastMarkerButtonView().addKeyListener(this.getCartoonMovie()._keyListener());

		this.addMouseListener(new MouseAdapter() {
			public void mouseEntered(MouseEvent e) {
				Component focusOwner = aWindow.getFocusOwner();
				if (focusOwner != null && focusOwner != JunCartoonMovieViewAwt.this) {
					JunCartoonMovieViewAwt.this.requestFocus();
				}
			}
		});

		aWindow.addWindowListener(new WindowAdapter() {
			public void windowActivated(WindowEvent e) {
				Component focusOwner = aWindow.getFocusOwner();
				if (focusOwner == null || focusOwner instanceof Container) {
					JunCartoonMovieViewAwt.this.requestFocus();
				}
			}
		});
	}

}
