package sra.jun.goodies.calendar;

import java.awt.Dialog;
import java.util.Calendar;
import sra.smalltalk.StBlockClosure;
import sra.smalltalk.StSimpleDialog;
import sra.smalltalk.StView;
import sra.jun.system.support.JunSystem;

/**
 * JunCalendarDateAndTimeDialog class
 * 
 *  @author    Nobuto Matsubara
 *  @created   2003/10/06 (by Nobuto Matsubara)
 *  @updated   N/A
 *  @version   445 (with StPL6.5) based on Jun472 for Smalltalk
 *  @copyright 1999-2004 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2004 Information-technology Promotion Agency, Japan (IPA)
 *  @copyright 2001-2004 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunCalendarDateAndTimeDialog.java,v 6.8 2004/04/15 06:27:55 nisinaka Exp $
 */
public class JunCalendarDateAndTimeDialog extends StSimpleDialog {
	protected JunCalendarDateAndTimeModel calendarDateAndTimeModel;
	protected StBlockClosure acceptBlock;
	protected StBlockClosure cancelBlock;

	/**
	 * Open a calendarDateAndTime dialog and return selected date and time.
	 *
	 * @return JunCalendarDateAndTimeModel
	 * @category utilities
	 */
	public static JunCalendarDateAndTimeModel Request() {
		return JunCalendarDateAndTimeDialog.Request_(null);
	}

	/**
	 * Open a calendarDateAndTime dialog and return selected date and time.
	 *
	 * @param messageString java.lang.String
	 * @return JunCalendarDateAndTimeModel
	 * @category utilities
	 */
	public static JunCalendarDateAndTimeModel Request_(String messageString) {
		Calendar[] dateAndTime = {Calendar.getInstance(), Calendar.getInstance()};
		return JunCalendarDateAndTimeDialog.Request_initialDateAndTime_(messageString, dateAndTime);
	}

	/**
	 * Initialize date and time , open a calendarDateAndTime dialog and return selected date and time.
	 *
	 * @param messageString java.lang.String
	 
	 * @return JunCalendarDateAndTimeModel
	 * @category utilities
	 */
	public static JunCalendarDateAndTimeModel Request_initialDateAndTime_(
		String messageString,
		Calendar[] dateAndTime) {
		return JunCalendarDateAndTimeDialog
			.Request_initialDateAndTime_onAccept_onCancel_(
			messageString,
			dateAndTime,
			null,
			null);
	}

	/**
	 * Initialize date and time , set accept button and cancel button,
	 * open a calendarDateAndTime dialog and return selected date and time.
	 *
	 * @param messageString java.lang.String
	 * @param acceptBlock sra.smalltalk.StBlockClosure
	 * @param cancelBlock sra.smalltalk.StBlockClosure
	 * @return JunCalendarDateAndTimeModel
	 * @category utilities
	 */
	public static JunCalendarDateAndTimeModel Request_initialDateAndTime_onAccept_onCancel_(
		String messageString,
		Calendar[] dateAndTime,
		StBlockClosure acceptBlock,
		StBlockClosure cancelBlock) {
		JunCalendarDateAndTimeDialog calendarDateAndTimeDialog =
			new JunCalendarDateAndTimeDialog();
		if (dateAndTime != null) {
			calendarDateAndTimeDialog.calendarDateAndTimeModel_(
				JunCalendarDateAndTimeModel.DateAndTime_(dateAndTime));
		}
		Dialog dialog = calendarDateAndTimeDialog.allButOpenDialog();
		if (messageString != null) {
			dialog.setTitle(messageString);
		}
		if (acceptBlock != null) {
			calendarDateAndTimeDialog.acceptBlock_(acceptBlock);
		}
		if (cancelBlock != null) {
			calendarDateAndTimeDialog.cancelBlock_(cancelBlock);
		}
		_ShowAtMousePoint(dialog);
		if (calendarDateAndTimeDialog.accept()._booleanValue()) {
			return calendarDateAndTimeDialog.calendarDateAndTimeModel();
		} else {
			return null;
		}
	}

	/**
	* Set an action accept to a block..
	* @param aBlock StBlockClosure
	* @category accessing
	*/
	public void acceptBlock_(StBlockClosure aBlock) {
		acceptBlock = aBlock;
	}

	/**
	 * Return aCalendarDateAndTimeModel.
	 *
	 * @return JunCalendarDateAndTimeModel
	 * @category accessing
	 */
	public JunCalendarDateAndTimeModel calendarDateAndTimeModel() {
		if (calendarDateAndTimeModel == null) {
			calendarDateAndTimeModel = new JunCalendarDateAndTimeModel();
		}
		return calendarDateAndTimeModel;
	}

	/**
	 * Set aCalendarDateAndTimeModel.
	 *
	 * @param aCalendarDateAndTimeModel JunCalendarDateAndTimeModel
	 * @category accessing
	 */
	public void calendarDateAndTimeModel_(JunCalendarDateAndTimeModel aCalendarDateAndTimeModel) {
		calendarDateAndTimeModel = aCalendarDateAndTimeModel;
	}

	/**
	* Set a calcel action to a block.
	* @param aBlock StBlockClosure
	* @category accessing
	*/
	public void cancelBlock_(StBlockClosure aBlock) {
		cancelBlock = aBlock;
	}

	/**
	* Return result of accept button pressed.
	* @return boolean
	* @category actions
	*/
	public Object acceptButtonPressed() {
		if (!(acceptBlock == null)) {
			return acceptBlock.value_(this.calendarDateAndTimeModel());
		}
		return new Boolean(true);
	}

	/**
	* Return result of cancel button pressed.
	* @return boolean
	* @category actions
	*/
	public Object cancelButtonPressed() {
		if (!(cancelBlock == null)) {
			return acceptBlock.value_(this.calendarDateAndTimeModel());
		}
		return new Boolean(true);
	}

	/**
	* Answer a default view.
	*
	* @return sra.smalltalk.StView
	* @category defaults
	*/
	public StView defaultView() {
		if (GetDefaultViewMode() == VIEW_AWT) {
			return new JunCalendarDateAndTimeDialogViewAwt(this);
		} else {
			return new JunCalendarDateAndTimeDialogViewSwing(this);
		}
	}

	/**
	* Answer a window title.
	*
	* @return java.lang.String
	* @category interface opening
	*/
	protected String windowTitle() {
		return JunSystem.$String("Date and Time Dialog");
	}
}
