package sra.jun.goodies.calendar;

import java.awt.Point;
import java.awt.event.*;

import sra.jun.system.framework.JunDialog;
import sra.jun.system.support.JunSystem;
import sra.smalltalk.*;

/**
 * JunCalendarDateController class
 * 
 *  @author    Nobuto Matsubara
 *  @created   2003/10/06 (by Nobuto Matsubara)
 *  @updated   N/A
 *  @version   445 (with StPL6.5) based on Jun472 for Smalltalk
 *  @copyright 1999-2004 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2004 Information-technology Promotion Agency, Japan (IPA)
 *  @copyright 2001-2004 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunCalendarDateController.java,v 6.7 2004/04/15 06:27:55 nisinaka Exp $
 */
public class JunCalendarDateController
	extends StController
	implements MouseListener {
	private MouseEvent mEvent;
	/**
     * Mouse button pressed event.
     * @param e MouseEvent
     * @category control defaults
     */
    public void mousePressed(MouseEvent e) {
		mEvent = e;
		if (mEvent.isMetaDown()) {
			this.view()._showPopupMenu(mEvent.getX(), mEvent.getY());
		} else {
			this.buttonActivity();
		}
	}

	/**
	 * Execute active button event
	 * @category activities
	 */
	public void buttonActivity() {
		Point point = new Point(mEvent.getX(), mEvent.getY());
		String key;
		StRectangle box;
		for (int i = 0; i < this.getView().frames().size(); ++i) {
			Object[] array = (Object[]) this.getView().frames().get(i);
			key = (String) array[0];
			box = (StRectangle) array[array.length - 1];
			if (box.containsPoint_(point)) {
				if (key.equals("year")) {
					this.yearActivity_(array);
				}
				if (key.equals("month")) {
					this.monthActivity_(array);
				}
				if (key.equals("weekday")) {
					this.weekdayActivity_(array);
				}
				if (key.equals("day")) {
					this.dayActivity_(array);
				}
				if (key.equals("button")) {
					this.buttonActivity_(array);
				}
			}
		}
	}

	/**
	 * Set active button event
	 * @category activities
	 */
	public void buttonActivity_(Object[] anArray) {
		String symbol = (String) anArray[1];
		if (symbol.equals("yearDown")) {
			this.getModel().yearDown();
		}
		if (symbol.equals("yearUp")) {
			this.getModel().yearUp();
		}
		if (symbol.equals("monthDown")) {
			this.getModel().monthDown();
		}
		if (symbol.equals("monthUp")) {
			this.getModel().monthUp();
		}
	}

	/**
	 * Set active button day event
	 * @category activities
	 */
	public void dayActivity_(Object[] anArray) {
		Point aPoint = (Point) anArray[1];
		int day = this.getModel().atPoint_(aPoint);
		this.getModel().day_(day);
	}
	
	/**
	 * Set active button month event
	 * @category activities
	 */
	public void monthActivity_(Object[] anArray) {
		String answer =
			JunDialog.Request_(
				JunSystem.$String("Month?"),
				String.valueOf(this.getModel().month() + 1));
		if (answer == null || answer.length() == 0) {
			return;
		}
		
		try {
			int month = Integer.parseInt(answer);
			this.getModel().month_(month - 1);
		} catch (NumberFormatException e) {
			JunDialog.Warn_(answer + JunSystem.$String(" is invalid value."));
		}
	}

	/**
	 * Set active button weekday event
	 * @category activities
	 */
	public void weekdayActivity_(Object[] anArray) {
		return;
	}

	/**
	 * Set active button year event
	 * @category activities
	 */
	public void yearActivity_(Object[] anArray) {
		String answer =
			JunDialog.Request_(JunSystem.$String("Year?"), String.valueOf(this.getModel().year()));
		if (answer == null || answer.length() == 0) {
			return;
		}
		
		try {
			int year = Integer.parseInt(answer);
			this.getModel().year_(year);
		} catch (NumberFormatException e) {
			JunDialog.Warn_(answer + JunSystem.$String(" is invalid value."));
		}
	}

	/**
	 * Return default view
	 * @category defaults
	 */
	public JunCalendarDateView getView() {
		return (JunCalendarDateView) this.view();
	}

	/**
	 * Return default model
	 * @category defaults
	 */
	public JunCalendarDateModel getModel() {
		return (JunCalendarDateModel) this.model();
	}
}
