package sra.jun.goodies.calendar;

import java.awt.Button;
import java.awt.Dialog;
import java.awt.Frame;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import sra.jun.system.support.JunSystem;
import sra.smalltalk.StSimpleDialogViewAwt;
import sra.smalltalk.WindowSpecLayout;

/**
 * JunCalendarDateDialogViewAwt class
 * 
 *  @author    Nobuto Matsubara
 *  @created   2003/10/06 (by Nobuto Matsubara)
 *  @updated   N/A
 *  @version   445 (with StPL6.5) based on Jun472 for Smalltalk
 *  @copyright 1999-2004 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2004 Information-technology Promotion Agency, Japan (IPA)
 *  @copyright 2001-2004 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunCalendarDateDialogViewAwt.java,v 6.8 2004/04/15 06:27:55 nisinaka Exp $
 */
public class JunCalendarDateDialogViewAwt
	extends StSimpleDialogViewAwt
	implements JunCalendarDateDialogView {
	protected JunCalendarDateViewAwt calendarDateView;
	protected Button acceptButton;
	protected Button cancelButton;

	/**
	 * Create a new instance of JunCaledarDateDialogViewAwt and initialize it with the JunCalendarDateAndTimeDialog.
	 * @param newCalendarDateAndTimeDialog JunCalendarDateAndTimeDialog
	 * @category instance creation
	 */
	public JunCalendarDateDialogViewAwt(JunCalendarDateDialog newCalendarDateDialog) {
		super(newCalendarDateDialog);
	}

	/**
	 * Answer the default dialog.
	 *
	 * @return java.awt.Dialog
	 * @category defaults
	 */
	public Dialog defaultDialog() {
		Dialog aDialog =
			new Dialog(
				new Frame(),
				this.getCalendarDateDialog().windowTitle(),
				true);
		aDialog.add(this);
		aDialog.pack();
		return aDialog;
	}

	/**
	 * Answer my model as a JunCalendarDateDialog.
	 * @return JunCalendarDateDialog
	 * @category accessing
	 */
	public JunCalendarDateDialog getCalendarDateDialog() {
		return (JunCalendarDateDialog) this.model();
	}

	/**
	 * Answer my accept button.
	 * @return java.awt.Button
	 * @category accessing
	 */
	protected Button acceptButton() {
		if (acceptButton == null) {
			acceptButton = new Button(JunSystem.$String("Accept"));
			acceptButton.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					if (((Boolean) getCalendarDateDialog()
						.acceptButtonPressed())
						.booleanValue()) {
						getCalendarDateDialog().accept().value_(true);
					}
				}
			});
		}
		return acceptButton;
	}

	/**
	 * Build this component.
	 * @category interface opening
	 */
	protected void buildComponent() {
		this.setLayout(new WindowSpecLayout());
		this.add(this.calendarDateView(), WindowSpecLayout.Position(0f, 1, 0f, 1, 1f, -102, 1f, -1));
		this.add(this.cancelButton(), WindowSpecLayout.Position(1f, -96, 0f, 56, 1f, -8, 0f, 88));
		this.add(this.acceptButton(), WindowSpecLayout.Position(1f, -96, 0f, 16, 1f, -8, 0f, 48));
		defaultButton = this.acceptButton();
		this.setSize(400, 200);
	}

	/**
	 * Answer my cancel button.
	 *
	 * @return java.awt.Button
	 * @category accessing
	 */
	protected Button cancelButton() {
		if (cancelButton == null) {
			cancelButton = new Button(JunSystem.$String("Cancel"));
			cancelButton.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					if (((Boolean) getCalendarDateDialog()
						.cancelButtonPressed())
						.booleanValue()) {
						getCalendarDateDialog().cancel().value_(true);
					}
				}
			});
		}
		return cancelButton;
	}

	/**
	 * Answer my calendar date view.
	 *
	 * @return JunCalendarDateViewAwt
	 * @category accessing
	 */
	protected JunCalendarDateViewAwt calendarDateView() {
		if (calendarDateView == null) {
			calendarDateView =
				new JunCalendarDateViewAwt(
					this.getCalendarDateDialog().calendarDateModel());
		}
		return calendarDateView;
	}
}
