package sra.jun.goodies.calendar;

import java.awt.Dialog;
import java.awt.Dimension;
import java.awt.Frame;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JButton;

import sra.jun.system.support.JunSystem;
import sra.smalltalk.StSimpleDialogViewSwing;
import sra.smalltalk.WindowSpecLayout;

/**
 * JunCalendarDateIntervalDialogViewSwing class
 * 
 *  @author    Nobuto Matsubara
 *  @created   2003/10/06 (by Nobuto Matsubara)
 *  @updated   N/A
 *  @version   445 (with StPL6.5) based on Jun472 for Smalltalk
 *  @copyright 1999-2004 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2004 Information-technology Promotion Agency, Japan (IPA)
 *  @copyright 2001-2004 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunCalendarDateIntervalDialogViewSwing.java,v 6.8 2004/04/15 06:27:55 nisinaka Exp $
 */
public class JunCalendarDateIntervalDialogViewSwing
	extends StSimpleDialogViewSwing
	implements JunCalendarDateIntervalDialogView {
	protected JunCalendarDateIntervalViewSwing calendarDateIntervalView;
	protected JButton acceptButton;
	protected JButton cancelButton;
	protected JunCalendarDateViewSwing fromView;
	protected JunCalendarDateViewSwing toView;

	/**
	 * Create a new instance of JunCaledarDateIntervalDialogViewSwing and initialize it with the JunCalendarDateIntervalDialog.
	 * @param newCalendarDateAndDateIntervalDialog JunCalendarDateIntervalDialog
	 * @category instance creation
	 */
	public JunCalendarDateIntervalDialogViewSwing(JunCalendarDateIntervalDialog newCalendarDateIntervalDialog) {
		super(newCalendarDateIntervalDialog);
	}

	/**
	 * Answer the default dialog.
	 *
	 * @return java.awt.Dialog
	 * @category defaults
	 */
	public Dialog defaultDialog() {
		Dialog aDialog =
			new Dialog(
				new Frame(),
				this.getCalendarDateIntervalDialog().windowTitle(),
				true);
		aDialog.add(this);
		aDialog.pack();
		return aDialog;
	}

	/**
	 * Answer my model as a JunCalendarDateIntervalDialog.
	 * @return JunCalendarDateIntervalDialog
	 * @category accessing
	 */
	public JunCalendarDateIntervalDialog getCalendarDateIntervalDialog() {
		return (JunCalendarDateIntervalDialog) this.model();
	}

	/**
	 * Answer my accept button.
	 * @return javax.swing.jButton
	 * @category accessing
	 */
	protected JButton acceptButton() {
		if (acceptButton == null) {
			acceptButton = new JButton(JunSystem.$String("Accept"));
			acceptButton.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					if (((Boolean) getCalendarDateIntervalDialog()
						.acceptButtonPressed())
						.booleanValue()) {
						getCalendarDateIntervalDialog().accept().value_(true);
					}
				}
			});
		}
		return acceptButton;
	}

	/**
	 * Build this component.
	 * @category interface opening
	 */
	protected void buildComponent() {
		this.setLayout(new WindowSpecLayout());
		this.add(this.fromView(), WindowSpecLayout.Position(0f, 1, 0f, 1, 1f, -102, 0.5f, -1));
		this.add(this.toView(), WindowSpecLayout.Position(0f, 1, 0.5f, 1, 1f, -102, 1f, -1));
		this.add(this.cancelButton(), WindowSpecLayout.Position(1f, -96, 0f, 56, 1f, -8, 0f, 88));
		this.add(this.acceptButton(), WindowSpecLayout.Position(1f, -96, 0f, 16, 1f, -8, 0f, 48));
		defaultButton = this.acceptButton();
		this.setPreferredSize(new Dimension(400, 400));
	}

	/**
	 * Answer my cancel button.
	 *
	 * @return javax.swing.jButton
	 * @category accessing
	 */
	protected JButton cancelButton() {
		if (cancelButton == null) {
			cancelButton = new JButton(JunSystem.$String("Cancel"));
			cancelButton.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					if (((Boolean) getCalendarDateIntervalDialog()
						.cancelButtonPressed())
						.booleanValue()) {
						getCalendarDateIntervalDialog().cancel().value_(true);
					}
				}
			});
		}
		return cancelButton;
	}

	/**
	 * Answer my calendar date view.
	 *
	 * @return JunCalendarDateIntervalViewSwing
	 * @category accessing
	 */
	protected JunCalendarDateIntervalViewSwing calendarDateIntervalView() {
		if (calendarDateIntervalView == null) {
			calendarDateIntervalView =
				new JunCalendarDateIntervalViewSwing(
					this
						.getCalendarDateIntervalDialog()
						.calendarDateIntervalModel());
		}
		return calendarDateIntervalView;
	}

	/**
	 * Answer from date view.
	 *
	 * @return JunCalendarDateViewSwing
	 * @category accessing
	 */
	protected JunCalendarDateViewSwing fromView() {
		if (fromView == null) {
			fromView =
				new JunCalendarDateViewSwing(
					this
						.getCalendarDateIntervalDialog()
						.calendarDateIntervalModel().fromCalendarDateModel());
		}
		return fromView;
	}

	/**
	 * Answer to date view.
	 *
	 * @return JunCalendarDateViewSwing
	 * @category accessing
	 */
	protected JunCalendarDateViewSwing toView() {
		if (toView == null) {
			toView =
				new JunCalendarDateViewSwing(
					this
						.getCalendarDateIntervalDialog()
						.calendarDateIntervalModel().toCalendarDateModel());
		}
		return toView;
	}
}
