package sra.jun.goodies.colors;

import java.awt.*;
import sra.smalltalk.*;
import sra.jun.system.support.JunSystem;

/**
 * JunColorChoiceDialog class
 * 
 *  @author    Mitsuhiro Asada
 *  @created   2003/04/02 (by Mitsuhiro Asada)
 *  @updated   N/A
 *  @version   445 (with StPL6.5) based on Jun490 for Smalltalk
 *  @copyright 1999-2004 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2004 Information-technology Promotion Agency, Japan (IPA)
 *  @copyright 2001-2004 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunColorChoiceDialog.java,v 6.7 2004/04/15 06:27:55 nisinaka Exp $
 */
public class JunColorChoiceDialog extends StSimpleDialog {
	protected JunColorChoiceModel colorChoiceModel;
	protected StBlockClosure acceptBlock;
	protected StBlockClosure cancelBlock;

	/**
	 * Open a color choice dialog and let a user to choose.
	 * 
	 * @return java.awt.Color
	 * @category Utilities
	 */
	public static Color Request() {
		return Request_(null);
	}

	/**
	 * Open a color choice dialog and let a user to choose.
	 * 
	 * @return java.awt.Color
	 * @param messageString java.lang.String
	 * @category Utilities
	 */
	public static Color Request_(String messageString) {
		return Request_colorChoice_(messageString, new JunColorChoiceSBH());
	}

	/**
	 * Open a color choice dialog and let a user to choose.
	 * 
	 * @return java.awt.Color
	 * @param messageString java.lang.String
	 * @param colorValue java.lang.Color
	 * @category Utilities
	 */
	public static Color Request_color_(String messageString, Color colorValue) {
		JunColorChoiceModel colorChoiceModel = new JunColorChoiceSBH();
		colorChoiceModel.color_(colorValue);
		return Request_colorChoice_onAccept_onCancel_(messageString, colorChoiceModel, null, null);
	}

	/**
	 * Open a color choice dialog and let a user to choose.
	 * 
	 * @return java.awt.Color
	 * @param messageString java.lang.String
	 * @param colorChoiceModel sra.jun.goodies.colors.JunColorChoiceModel
	 * @category Utilities
	 */
	public static Color Request_colorChoice_(String messageString, JunColorChoiceModel colorChoiceModel) {
		return Request_colorChoice_onAccept_onCancel_(messageString, colorChoiceModel, null, null);
	}

	/**
	 * Open a color choice dialog and let a user to choose.
	 * 
	 * @return java.awt.Color
	 * @param messageString java.lang.String
	 * @param colorChoiceModel sra.jun.goodies.colors.JunColorChoiceModel
	 * @param acceptBlock sra.smalltalk.StBlockClosure
	 * @param cancelBlock sra.smalltalk.StBlockClosure
	 * @category Utilities
	 */
	public static Color Request_colorChoice_onAccept_onCancel_(String messageString, JunColorChoiceModel colorChoiceModel, StBlockClosure acceptBlock, StBlockClosure cancelBlock) {
		JunColorChoiceDialog colorChoiceDialog = new JunColorChoiceDialog();
		if (colorChoiceModel != null) {
			colorChoiceDialog.colorChoiceModel_(colorChoiceModel);
		}
		Dialog dialog = colorChoiceDialog.allButOpenDialog();
		if (messageString != null) {
			dialog.setTitle(messageString);
		}
		if (acceptBlock != null) {
			colorChoiceDialog.acceptBlock_(acceptBlock);
		}
		if (cancelBlock != null) {
			colorChoiceDialog.cancelBlock_(cancelBlock);
		}

		_ShowAtMousePoint(dialog);
		if (colorChoiceDialog.accept()._booleanValue()) {
			return colorChoiceDialog.colorChoiceModel().color();
		} else {
			return null;
		}
	}

	/**
	 * Called when an accept button is pressed.
	 *
	 * @return boolean
	 * @category actions
	 */
	public boolean acceptButtonPressed() {
		if (acceptBlock != null) {
			return ((Boolean) acceptBlock.value_(this.colorChoiceModel())).booleanValue();
		}
		return true;
	}

	/**
	 * Called when an cancel button is pressed.
	 *
	 * @return boolean
	 * @category actions
	 */
	public boolean cancelButtonPressed() {
		if (cancelBlock != null) {
			return ((Boolean) cancelBlock.value_(this.colorChoiceModel())).booleanValue();
		}
		return true;
	}

	/**
	 * Answer the current color choice model.
	 *
	 * @return sra.jun.goodies.colors.JunColorChoiceModel
	 * @category accessing
	 */
	public JunColorChoiceModel colorChoiceModel() {
		if (colorChoiceModel == null) {
			colorChoiceModel = new JunColorChoiceSBH();
		}
		return colorChoiceModel;
	}

	/**
	 * Set the current color choice model.
	 *
	 * @param coiceModel sra.jun.goodies.colors.JunColorChoiceModel
	 * @category accessing
	 */
	public void colorChoiceModel_(JunColorChoiceModel choiceModel) {
		colorChoiceModel = choiceModel;
	}

	/**
	 * Answer a default view.
	 * 
	 * @return sra.smalltalk.StView
	 * @category interface opening
	 */
	public StView defaultView() {
		if (GetDefaultViewMode() == VIEW_AWT) {
			return new JunColorChoiceDialogViewAwt(this);
		} else {
			return new JunColorChoiceDialogViewSwing(this);
		}
	}

	/**
	 * Set the new accept block.
	 *
	 * @param newAcceptBlock sra.smalltalk.StBlockClosure
	 * @category accessing
	 */
	protected void acceptBlock_(StBlockClosure newAcceptBlock) {
		acceptBlock = newAcceptBlock;
	}

	/**
	 * Set the new cancel block.
	 *
	 * @param newCancelBlock sra.smalltalk.StBlockClosure
	 * @category accessing
	 */
	protected void cancelBlock_(StBlockClosure newCancelBlock) {
		cancelBlock = newCancelBlock;
	}

	/**
	 * Initialize the ApplicationModel when created.
	 * 
	 * @category initialize-release
	 */
	protected void initialize() {
		super.initialize();

		acceptBlock = null;
		cancelBlock = null;
	}

	/**
	 * Answer a window title.
	 * 
	 * @return java.lang.String
	 * @category interface opening
	 */
	protected String windowTitle() {
		return JunSystem.$String("Color Choice Dialog");
	}
}