package sra.jun.goodies.files;

import java.io.File;
import sra.smalltalk.*;
import sra.jun.system.support.JunSystem;
import sra.jun.system.support.JunTestExamples;

/**
 * JunFileDialogTestExamples class
 * 
 *  @author    nisinaka
 *  @created   2000/12/18 (by nisinaka)
 *  @updated   2003/03/09 (by nisinaka)
 *  @version   445 (with StPL6.5) based on Jun373 for Smalltalk
 *  @copyright 1999-2004 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2004 Information-technology Promotion Agency, Japan (IPA)
 *  @copyright 2001-2004 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunFileDialogTestExamples.java,v 6.7 2004/04/15 06:27:56 nisinaka Exp $
 */
public class JunFileDialogTestExamples extends JunTestExamples {

	/**
	 * Example_RequestNewDirectory: Call Request().
	 * 
	 * @return boolean
	 */
	public static boolean Example_RequestNewDirectory() {
		File answer = JunFileDialog.RequestNewDirectory(JunSystem.$String("Select a directory."), new File("Jun999"));
		System.out.println(answer);
		return true;
	}

	/**
	 * Example_RequestNewFilename: Call Request().
	 * 
	 * @return boolean
	 */
	public static boolean Example_RequestNewFilename() {
		File answer = JunFileDialog.RequestNewFilename(JunSystem.$String("Select a file."), new File("Jun999"));
		System.out.println(answer);
		return true;
	}

	/**
	 * Example1: Call Request().
	 * 
	 * @return boolean
	 */
	public static boolean Example1() {
		File answer = JunFileDialog.Request();
		System.out.println(answer);
		return true;
	}

	/**
	 * Example2: Request to select a file.
	 * 
	 * @return boolean
	 */
	public static boolean Example2() {
		File answer = JunFileDialog.Request(JunSystem.$String("Select a file."));
		System.out.println(answer);
		return true;
	}

	/**
	 * Example3: Request to select a file.
	 * 
	 * @return boolean
	 */
	public static boolean Example3() {
		File answer = JunFileDialog.Request(null, new File(JunFileModel.DefaultDirectory(), "zzz.st"), null, null, null, null);
		System.out.println(answer);
		return true;
	}

	/**
	 * Example4: Request to select a file.
	 * 
	 * @return boolean
	 */
	public static boolean Example4() {
		JunFileModel.FileType[] fileTypes =
			new JunFileModel.FileType[] { JunFileModel.FileType.All(JunSystem.$String("All files")), new JunFileModel.FileType(JunSystem.$String("Java files"), new String[] { "*.java", "*.JAVA" }), new JunFileModel.FileType(JunSystem.$String("Text files"), new String[] { "*.txt", "*.TXT" })
		};
		File answer = JunFileDialog.Request(JunSystem.$String("Select a file."), fileTypes, fileTypes[2]);
		System.out.println(answer);
		return true;
	}

	/**
	 * Example5: Request to select a file.
	 * 
	 * @return boolean
	 */
	public static boolean Example5() {
		JunFileModel.FileType[] fileTypes =
			new JunFileModel.FileType[] { JunFileModel.FileType.All(JunSystem.$String("All files")), new JunFileModel.FileType(JunSystem.$String("Java files"), new String[] { "*.java", "*.JAVA" }), new JunFileModel.FileType(JunSystem.$String("Text files"), new String[] { "*.txt", "*.TXT" })
		};
		File answer = JunFileDialog.Request(JunSystem.$String("Select a file."), JunFileModel.DefaultDirectory(), "zzz.java", fileTypes, fileTypes[2], new StBlockClosure() {
			public Object value_(Object model) {
				System.out.println(JunSystem.$String("Accept button was pressed."));
				System.out.println("\t" + ((JunFileModel) model).value());
				return Boolean.TRUE;
			}
		}, new StBlockClosure() {
			public Object value_(Object model) {
				System.out.println(JunSystem.$String("Cancel button was pressed."));
				System.out.println("\t" + ((JunFileModel) model).value());
				return Boolean.TRUE;
			}
		});
		System.out.println(answer);
		return true;
	}

	/**
	 * Execute all examples.
	 * 
	 * @param args an array of command-line arguments
	 */
	public static void main(java.lang.String[] args) {
		new JunFileDialogTestExamples();
	}
}
