package sra.jun.goodies.font;


import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import javax.swing.JButton;
import sra.smalltalk.StSimpleDialogViewSwing;
import sra.smalltalk.WindowSpecLayout;
import sra.jun.system.support.JunSystem;

/**
 * JunFontDialogViewSwing class
 * 
 *  @author    nisinaka
 *  @created   2003/02/28 (by nisinaka)
 *  @updated   N/A
 *  @version   445 (with StPL6.5) based on Jun433 for Smalltalk
 *  @copyright 1999-2004 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2004 Information-technology Promotion Agency, Japan (IPA)
 *  @copyright 2001-2004 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunFontDialogViewSwing.java,v 6.6 2004/04/15 06:27:57 nisinaka Exp $
 */
public class JunFontDialogViewSwing extends StSimpleDialogViewSwing implements JunFontDialogView {

	protected JunFontViewSwing fontView;
	protected JButton acceptButton;
	protected JButton cancelButton;

	/**
	 * Create a new instance of JunFontDialogViewSwing and initialize it with the JunFontDialog.
	 *
	 * @param newFontDialog sra.jun.goodies.font.JunFontDialog
	 */
	public JunFontDialogViewSwing(JunFontDialog newFontDialog) {
		super(newFontDialog);
	}

	/**
	 * Answer my model as a JunFontDialog.
	 *
	 * @return sra.jun.goodies.font.JunFontDialog
	 */
	public JunFontDialog getFontDialog() {
		return (JunFontDialog) this.model();
	}

	/**
	 * Answer my accpet button.
	 *
	 * @return javax.swing.JButton
	 */
	protected JButton acceptButton() {
		if (acceptButton == null) {
			acceptButton = new JButton(JunSystem.$String("Accept"));
			acceptButton.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					if (getFontDialog().acceptButtonPressed()) {
						getFontDialog().accept().value_(true);
					}
				}
			});
		}
		return acceptButton;
	}

	/**
	 * Build this component.
	 */
	protected void buildComponent() {
		this.setLayout(new WindowSpecLayout());
		this.add(this.fontView(), WindowSpecLayout.Position(0f, 1, 0f, 1, 1f, -1, 1f, -50));
		this.add(this.acceptButton(), WindowSpecLayout.Position(0.3f, -44, 1f, -40, 0.3f, 44, 1f, -8));
		this.add(this.cancelButton(), WindowSpecLayout.Position(0.7f, -44, 1f, -40, 0.7f, 44, 1f, -8));
		this.setPreferredSize(new Dimension(350, 250));

		defaultButton = this.acceptButton();
	}

	/**
	 * Answer my cancel button.
	 *
	 * @return javax.swing.JButton
	 */
	protected JButton cancelButton() {
		if (cancelButton == null) {
			cancelButton = new JButton(JunSystem.$String("Cancel"));
			cancelButton.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					if (getFontDialog().cancelButtonPressed()) {
						getFontDialog().cancel().value_(true);
					}
				}
			});
		}
		return cancelButton;
	}

	/**
	 * Answer my font view.
	 *
	 * @return sra.jun.goodies.font.JunFontViewSwing
	 */
	protected JunFontViewSwing fontView() {
		if (fontView == null) {
			fontView = new JunFontViewSwing(this.getFontDialog().fontModel());
		}
		return fontView;
	}
}
