package sra.jun.goodies.itemlist;

import java.awt.Dialog;
import sra.smalltalk.StBlockClosure;
import sra.smalltalk.StView;
import sra.jun.system.support.JunSystem;

/**
 * JunChoiceListDialog class
 * 
 *  @author    Mitsuhiro Asada
 *  @created   2003/12/15 (by Mitsuhiro Asada)
 *  @updated   N/A
 *  @version   445 (with StPL6.5) based on Jun465 for Smalltalk
 *  @copyright 1999-2004 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2004 Information-technology Promotion Agency, Japan (IPA)
 *  @copyright 2001-2004 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunChoiceListDialog.java,v 1.7 2004/04/15 06:27:58 nisinaka Exp $
 */
public class JunChoiceListDialog extends JunItemListDialog {

	/**
	 * Create a new instance of JunChoiceListDialog.
	 * 
	 * @category Instance creation
	 */
	public JunChoiceListDialog() {
		super();
	}

	/**
	 * Open a JunChoiceListDialog showing a message string and let a user to select item value.
	 * 
	 * @return java.lang.Object[]
	 * @param messageString java.lang.String
	 * @param stringCollection java.lang.String[]
	 * @category Utilities
	 */
	public static Object[] Request_labels_(String messageString, String[] stringCollection) {
		return Request_labels_onAccept_onCancel_(messageString, stringCollection, null, null);
	}

	/**
	 * Open a JunChoiceListDialog showing a message string and let a user to select item value.
	 * 
	 * @return java.lang.Object[]
	 * @param messageString java.lang.String
	 * @param stringCollection java.lang.String[]
	 * @param acceptBlock sra.smalltalk.StBlockClosure
	 * @param cancelBlock sra.smalltalk.StBlockClosure
	 * @category Utilities
	 */
	public static Object[] Request_labels_onAccept_onCancel_(String messageString, String[] stringCollection, StBlockClosure acceptBlock, StBlockClosure cancelBlock) {
		return Request_labels_checks_onAccept_onCancel_(messageString, stringCollection, null, acceptBlock, cancelBlock);
	}

	/**
	 * Open a JunChoiceListDialog showing a message string and let a user to select item value.
	 * 
	 * @return java.lang.Object[]
	 * @param messageString java.lang.String
	 * @param stringCollection java.lang.String[]
	 * @param booleanCollection boolean[]
	 * @category Utilities
	 */
	public static Object[] Request_labels_checks_(String messageString, String[] stringCollection, boolean[] booleanCollection) {
		return Request_labels_checks_onAccept_onCancel_(messageString, stringCollection, booleanCollection, null, null);
	}

	/**
	 * Open a JunChoiceListDialog showing a message string and let a user to select item value.
	 * 
	 * @return java.lang.Object[]
	 * @param messageString java.lang.String
	 * @param stringCollection java.lang.String[]
	 * @param checkBooleanCollection boolean[]
	 * @param lockBooleanCollection boolean[]
	 * @category Utilities
	 */
	public static Object[] Request_labels_checks_locks_(String messageString, String[] stringCollection, boolean[] checkBooleanCollection, boolean[] lockBooleanCollection) {
		return Request_labels_checks_locks_onAccept_onCancel_(messageString, stringCollection, checkBooleanCollection, lockBooleanCollection, null, null);
	}

	/**
	 * Open a JunChoiceListDialog showing a message string and let a user to select item value.
	 * 
	 * @return java.lang.Object[]
	 * @param messageString java.lang.String
	 * @param stringCollection java.lang.String[]
	 * @param booleanCollection boolean[]
	 * @param acceptBlock sra.smalltalk.StBlockClosure
	 * @param cancelBlock sra.smalltalk.StBlockClosure
	 * @category Utilities
	 */
	public static Object[] Request_labels_checks_onAccept_onCancel_(String messageString, String[] stringCollection, boolean[] booleanCollection, StBlockClosure acceptBlock, StBlockClosure cancelBlock) {
		return Request_labels_checks_locks_onAccept_onCancel_(messageString, stringCollection, booleanCollection, null, acceptBlock, cancelBlock);
	}

	/**
	 * Open a JunChoiceListDialog showing a message string and let a user to select item value.
	 * 
	 * @return java.lang.Object[]
	 * @param messageString java.lang.String
	 * @param stringCollection java.lang.String[]
	 * @param checkBooleanCollection boolean[]
	 * @param lockBooleanCollection boolean[]
	 * @param acceptBlock sra.smalltalk.StBlockClosure
	 * @param cancelBlock sra.smalltalk.StBlockClosure
	 * @category Utilities
	 */
	public static Object[] Request_labels_checks_locks_onAccept_onCancel_(String messageString, String[] stringCollection, boolean[] checkBooleanCollection, boolean[] lockBooleanCollection, StBlockClosure acceptBlock, StBlockClosure cancelBlock) {
		JunChoiceListDialog itemListDialog = new JunChoiceListDialog();
		itemListDialog.itemListModel_(new JunChoiceListModel(stringCollection, checkBooleanCollection, lockBooleanCollection));

		Dialog dialog = itemListDialog.allButOpenDialog();
		if (messageString != null) {
			dialog.setTitle(messageString);
		}
		if (acceptBlock != null) {
			itemListDialog.acceptBlock_(acceptBlock);
		}
		if (cancelBlock != null) {
			itemListDialog.cancelBlock_(cancelBlock);
		}
		_ShowAtMousePoint(dialog);

		if (itemListDialog.accept()._booleanValue()) {
			return itemListDialog.itemListModel().value();
		} else {
			return null;
		}
	}

	/**
	 * Answer a window title.
	 *
	 * @return java.lang.String
	 * @see sra.smalltalk.StApplicationModel#windowTitle()
	 * @category viewing
	 */
	protected String windowTitle() {
		return JunSystem.$String("Choice List");
	}

	/**
	 * Answer a default view.
	 * 
	 * @return sra.smalltalk.StView
	 * @category defaults
	 */
	public StView defaultView() {
		if (GetDefaultViewMode() == VIEW_AWT) {
			return new JunChoiceListDialogViewAwt(this);
		} else {
			return new JunChoiceListDialogViewSwing(this);
		}
	}

}
