package sra.jun.goodies.itemlist;

import java.awt.*;
import java.awt.image.BufferedImage;
import java.util.Vector;
import javax.swing.*;
import sra.smalltalk.*;
import sra.jun.goodies.button.JunButtonViewSwing;
import sra.jun.goodies.cursors.JunCursors;

/**
 * JunItemListViewSwing class
 * 
 *  @author    Mitsuhiro Asada
 *  @created   2003/12/12 (by Mitsuhiro Asada)
 *  @updated   N/A
 *  @version   445 (with StPL6.5) based on Jun465 for Smalltalk
 *  @copyright 1999-2004 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2004 Information-technology Promotion Agency, Japan (IPA)
 *  @copyright 2001-2004 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunItemListViewSwing.java,v 1.6 2004/04/15 06:27:58 nisinaka Exp $
 */
public class JunItemListViewSwing extends JunOrderListViewSwing implements JunItemListView {
	protected Vector checkMarkRectangles;

	protected JunButtonViewSwing checkOffButtonView;
	protected JunButtonViewSwing checkOnButtonView;


	/**
	 * Create a new instance of JunItemListViewSwing.
	 * 
	 * @category Instance creation
	 */
	public JunItemListViewSwing() {
		super();
	}

	/**
	 * Create a new instance of JunItemListViewSwing.
	 * 
	 * @param newModel sra.smalltalk.StModel
	 * @category Instance creation
	 */
	public JunItemListViewSwing(StModel newModel) {
		super(newModel);
	}

	/**
	 * Create a new view for the JunItemListViewSwing.
	 * The view consists of a JunItemListViewSwing on a ScrollPane.
	 * The view having up/down buttons.
	 * 
	 * @return sra.smalltalk.StView
	 * @param newModel sra.jun.goodies.itemlist.JunOrderListModel
	 * @category Instance creation
	 */
	public static StView OnScrollPaneWithButtons(JunOrderListModel newModel) {
		return new StViewJPanel(newModel) {
			protected JunItemListViewSwing itemListView;

			protected void buildComponent() {
				itemListView = new JunItemListViewSwing((JunOrderListModel) this.model());
				JScrollPane scrollPane = new JScrollPane(itemListView, JScrollPane.VERTICAL_SCROLLBAR_ALWAYS, JScrollPane.HORIZONTAL_SCROLLBAR_ALWAYS) {
					public void setBounds(int x, int y, int width, int height) {
						super.setBounds(x, y, width, height);
						this.doLayout();
						itemListView.setBounds(0, 0, this.getViewport().getExtentSize().width, this.getViewport().getExtentSize().height);
					}
				};

				this.setLayout(new WindowSpecLayout());
				this.add(scrollPane, WindowSpecLayout.Position(0f, 1, 0f, 1, 1f, -20, 1f, -1));
				this.add(itemListView.upButtonView(), WindowSpecLayout.Position(1f, -19, 0.3f, -19, 1f, -1, 0.3f, -1));
				this.add(itemListView.downButtonView(), WindowSpecLayout.Position(1f, -19, 0.3f, 1, 1f, -1, 0.3f, 19));
				this.add(itemListView.checkOnButtonView(), WindowSpecLayout.Position(1f, -19, 0.7f, -19, 1f, -1, 0.7f, -1));
				this.add(itemListView.checkOffButtonView(), WindowSpecLayout.Position(1f, -19, 0.7f, 1, 1f, -1, 0.7f, 19));
				this.setPreferredSize(new Dimension(150, 150));
			}
		};
	}

	/**
	 * Answer my check mark rectangles.
	 * 
	 * @return java.util.Vector
	 * @category accessing
	 */
	public Vector checkMarkRectangles() {
		if (checkMarkRectangles == null) {
			checkMarkRectangles = new Vector();
		}
		return checkMarkRectangles;
	}

	/**
	 * Answer my left margin size.
	 * 
	 * @return int
	 * @category accessing
	 */
	public int leftMargin() {
		return 23;
	}

	/**
	 * Answer my default controller.
	 * 
	 * @return sra.smalltalk.StController
	 * @category controller accessing
	 */
	protected StController defaultController() {
		return new JunItemListController();
	}

	/**
	 * Answer my model as a JunItemListModel.
	 * 
	 * @return sra.jun.goodies.itemlist.JunItemListModel
	 * @see sra.jun.goodies.itemlist.JunItemListView#getItemListModel()
	 * @category model accessing
	 */
	public JunItemListModel getItemListModel() {
		return (JunItemListModel) this.model();
	}

	/**
	 * Answer my check off button view.
	 *
	 * @return sra.jun.goodies.button.JunButtonViewSwing
	 * @category interface opening
	 */
	protected JunButtonViewSwing checkOffButtonView() {
		if (checkOffButtonView == null) {
			checkOffButtonView = new JunButtonViewSwing(this.getItemListModel().checkOffButton());
		}
		return checkOffButtonView;
	}

	/**
	 * Answer my check on button view.
	 *
	 * @return sra.jun.goodies.button.JunButtonViewSwing
	 * @category interface opening
	 */
	protected JunButtonViewSwing checkOnButtonView() {
		if (checkOnButtonView == null) {
			checkOnButtonView = new JunButtonViewSwing(this.getItemListModel().checkOnButton());
		}
		return checkOnButtonView;
	}

	/**
	 * Paints this component.
	 * 
	 * @param aGraphicsContext java.awt.Graphics
	 * @see sra.smalltalk.StViewCanvas#displayOn_(Graphics)
	 * @category displaying
	 */
	public void displayOn_(Graphics aGraphicsContext) {
		if (this.isShowing() == false) {
			return;
		}

		Rectangle clippingBounds = aGraphicsContext.getClipBounds();
		if (clippingBounds == null) {
			clippingBounds = this.getBounds();
		}
		Graphics graphicsContext = aGraphicsContext.create();
		try {
			graphicsContext.setColor(this.getBackground());
			graphicsContext.fillRect(clippingBounds.x, clippingBounds.y, clippingBounds.width, clippingBounds.height);
			if (this.getModel().isEmpty()) {
				super.displayOn_(graphicsContext);
				return;
			}
			for (int anIndex = 0; anIndex < this.getModel().size(); anIndex++) {
				JunItemListObject itemObject = (JunItemListObject) this.itemObjectAt_(anIndex);
				Rectangle rowBox = this.rowBoundsAt_(anIndex);
				rowBox = new Rectangle(rowBox.x + this.leftMargin(), rowBox.y, rowBox.width - this.leftMargin(), rowBox.height);
				if (this.getModel().isSelected_(anIndex)) {
					graphicsContext.setColor(this.selectionBackgroundColor());
					graphicsContext.fillRect(rowBox.x, rowBox.y, rowBox.width, rowBox.height);
				}

				Rectangle checkMarkBox = (Rectangle) this.checkMarkRectangles().get(anIndex);
				BufferedImage opaqueImage = itemObject.checkMark() ? JunCursors.CheckOnCursorImage() : JunCursors.CheckOffCursorImage();
				graphicsContext.drawImage(opaqueImage, checkMarkBox.x, checkMarkBox.y, this);

				StComposedText composedText = new StComposedText(itemObject.itemString());
				if (this.getModel().isSelected_(anIndex)) {
					graphicsContext.setColor(this.selectionForegroundColor());
				} else {
					graphicsContext.setColor(this.getForeground());
				}
				composedText.displayOn_at_(graphicsContext, new Point(rowBox.x + this.textMargin(), (rowBox.y + rowBox.height / 2) - composedText.height() / 2));
			}
		} finally {
			if (graphicsContext != null) {
				graphicsContext.dispose();
				graphicsContext = null;
			}
		}
	}

	/**
	 * Answer the index position which item object rectangle contains a point.
	 * 
	 * @return int
	 * @param aPoint java.awt.Point
	 * @see sra.jun.goodies.itemlist.JunItemListView#whichOfCheckMarksAt_(java.awt.Point)
	 * @category selecting
	 */
	public int whichOfCheckMarksAt_(Point aPoint) {
		Rectangle[] boxes = this._checkMarkRectangleArray();
		for (int anIndex = 0; anIndex < boxes.length; anIndex++) {
			if (boxes[anIndex].contains(aPoint)) {
				return anIndex;
			}
		}
		return -1;
	}

	/**
	 * Flush cached information.
	 * 
	 * @category private
	 */
	protected void flushCachedInformation() {
		super.flushCachedInformation();
		checkMarkRectangles = null;
	}

	/**
	 * Compute cached information.
	 * 
	 * @category private
	 */
	protected void computeCachedInformation() {
		checkMarkRectangles = new Vector();
		itemVisualRectangles = new Vector();
		itemRowRectangles = new Vector();
		boundingBox = new Rectangle(0, 0, 0, 0);
		if (this.getModel().isEmpty()) {
			return;
		}
		final int lineGrid = this.getModel().lineGrid();
		final int leftMargin = this.leftMargin();
		final Point aPoint = new Point(0, 0);
		this.getModel().do_(new StBlockClosure() {
			public Object value_(Object anObject) {
				JunItemListObject itemObject = (JunItemListObject) anObject;

				Rectangle aBox = new Rectangle(aPoint.x, aPoint.y, leftMargin, lineGrid);
				BufferedImage opaqueImage = (itemObject.checkMark()) ? JunCursors.CheckOnCursorImage() : JunCursors.CheckOffCursorImage();
				aBox = new Rectangle((aBox.x + aBox.width / 2) - (opaqueImage.getWidth() / 2), (aBox.y + aBox.height / 2) - (opaqueImage.getHeight() / 2), opaqueImage.getWidth(), opaqueImage.getHeight());
				JunItemListViewSwing.this.checkMarkRectangles.add(aBox);
				JunItemListViewSwing.this.boundingBox.add(aBox);

				aBox = new Rectangle(aPoint.x + leftMargin, aPoint.y, JunItemListViewSwing.this.getBounds().width - leftMargin, lineGrid);
				StComposedText composedText = new StComposedText(itemObject.itemString());
				aBox = new Rectangle(aBox.x + JunItemListViewSwing.this.textMargin() * 2, (aBox.y + aBox.height / 2) - (composedText.height() / 2), composedText.width(), composedText.height());
				JunItemListViewSwing.this.itemVisualRectangles.add(aBox);
				JunItemListViewSwing.this.boundingBox.add(aBox);

				aBox = new Rectangle(aPoint.x, aPoint.y, JunItemListViewSwing.this.getBounds().width, lineGrid);
				JunItemListViewSwing.this.itemRowRectangles.add(aBox);
				JunItemListViewSwing.this.boundingBox.add(aBox);

				aPoint.translate(0, lineGrid);
				return null;
			}
		});

		Rectangle[] rowRects = this._itemRowRectangleArray();
		for (int i = 0; i < rowRects.length; i++) {
			rowRects[i].width = boundingBox.width - boundingBox.x;
		}
	}
}
