package sra.jun.goodies.lisp;

import java.io.*;
import java.awt.*;
import java.awt.event.*;
import sra.smalltalk.*;

/**
 * JunLispInterpreterViewAwt class
 * 
 *  @author    nisinaka
 *  @created   2003/05/07 (by nisinaka)
 *  @updated   N/A
 *  @version   445 (with StPL6.5) based on Jun490 for Smalltalk
 *  @copyright 1999-2004 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2004 Information-technology Promotion Agency, Japan (IPA)
 *  @copyright 2001-2004 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunLispInterpreterViewAwt.java,v 6.6 2004/04/15 06:27:59 nisinaka Exp $
 */
public class JunLispInterpreterViewAwt extends StViewPanel implements JunLispInterpreterView {

	protected TextArea textValueView;
	protected TextArea textCollectorView;

	/**
	 * Create a new instance of JunLispInterpreterViewAwt and initialize it.
	 *
	 * @param aLispInterpreter sra.jun.goodies.lisp.JunLispInterpreter
	 */
	public JunLispInterpreterViewAwt(JunLispInterpreter aLispInterpreter) {
		super(aLispInterpreter);
	}

	/**
	 * Answer my model as JunLispInterpreter.
	 *
	 * @return sra.jun.goodies.lisp.JunLispInterpreter
	 */
	public JunLispInterpreter getLispInterpreter() {
		return (JunLispInterpreter) this.model();
	}

	/**
	 * Answer the selected text to 'do it'.
	 *
	 * @return java.lang.String
	 */
	public String getSelectedText() {
		return this.textValueView().getSelectedText();
	}

	/**
	 * Build this component.
	 */
	protected void buildComponent() {
		this.setLayout(new WindowSpecLayout());
		this.add(this.textValueView(), WindowSpecLayout.Position(0f, 1, 0f, 1, 1f, -1, 0.3f, -1));
		this.add(this.textCollectorView(), WindowSpecLayout.Position(0f, 1, 0.3f, 0, 1f, -1, 1f, -1));
		this.setSize(350, 400);
	}

	/**
	 * Answer the view of my text collector.
	 *
	 * @return java.awt.TextArea
	 */
	protected TextArea textCollectorView() {
		if (textCollectorView == null) {
			textCollectorView = new TextArea("", 0, 0, TextArea.SCROLLBARS_VERTICAL_ONLY);
			textCollectorView.setBackground(Color.lightGray);

			ByteArrayOutputStream aByteArrayOutputStream = new ByteArrayOutputStream() {
				public synchronized void write(byte b[], int off, int len) {
					super.write(b, off, len);
					textCollectorView.append(new String(b, off, len));
				}
			};
			this.getLispInterpreter().textCollector_(new PrintStream(aByteArrayOutputStream));
		}
		return textCollectorView;
	}

	/**
	 * Answer the view of my text value.
	 *
	 * @return java.awt.TextArea
	 */
	protected TextArea textValueView() {
		if (textValueView == null) {
			textValueView = new TextArea((String) this.getLispInterpreter().textValue().value(), 0, 0, TextArea.SCROLLBARS_VERTICAL_ONLY);

			StMenu questionMenu = this.getLispInterpreter().questionMenu();
			StMenuItem doIt = questionMenu.getItem("do it");
			doIt.setActionArgument(this);

			final PopupMenu aPopupMneu = this._createPopupMenu(questionMenu);
			textValueView.add(aPopupMneu);
			textValueView.addMouseListener(new MouseAdapter() {
				public void mousePressed(MouseEvent e) {
					if (e.isMetaDown()) {
						aPopupMneu.show(textValueView, e.getX(), e.getY());
					}
				}
			});

			this.getLispInterpreter().textValue().compute_(new StBlockClosure() {
				public Object value_(Object newValue) {
					textValueView.setText((String) newValue);
					return null;
				}
			});
		}
		return textValueView;
	}
}
