package sra.jun.goodies.movie;

import java.awt.*;
import java.io.*;

/**
 * JunMovieHandleWithMovieController class
 * 
 *  @author    Hoshi Takanori
 *  @created   2003/12/05 (by Hoshi Takanori)
 *  @updated   N/A
 *  @version   445 (with StPL6.5) based on JunXXX for Smalltalk
 *  @copyright 1999-2004 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2004 Information-technology Promotion Agency, Japan (IPA)
 *  @copyright 2001-2004 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunMovieHandleWithMovieController.java,v 6.7 2004/04/15 06:27:59 nisinaka Exp $
 */
public class JunMovieHandleWithMovieController extends JunMovieHandle {
	protected int movieController;

	/**
	 * get a JunMovieHandleWithMovieController with file.
	 * 
	 * @param aFilename java.io.File
	 * 
	 * @return sra.jun.goodies.movie.JunMovieHandle
	 */
	public static JunMovieHandle Filename_(File aFilename) {
		if ((aFilename == null) || !aFilename.exists()) {
			return null;
		}

		JunMovieHandle aMovieHandle = new JunMovieHandleWithMovieController();
		aMovieHandle.setFilename_(aFilename);
		aMovieHandle.setMovie_(aMovieHandle._interface().openMovie(aFilename));

		return aMovieHandle;
	}

	/**
	 * get a JunMovieHandleWithMovieController with file.
	 * 
	 * @param aFilename java.lang.String
	 * 
	 * @return sra.jun.goodies.movie.JunMovieHandle
	 */
	public static JunMovieHandle Filename_(String aFilename) {
		return JunMovieHandleWithMovieController.Filename_(new File(aFilename));
	}

	/**
	 * open the movie and return the movie handle.
	 * 
	 * @return sra.jun.goodies.movie.JunMovieHandle
	 */
	public static JunMovieHandle OpenMovie() {
		JunMovieHandle movieHandle = new JunMovieHandleWithMovieController();
		movieHandle.openMovie();

		return movieHandle;
	}

	/**
	 * do movie task.
	 */
	public void doMovieTask() {
		if (this.hasMovieController() == true) {
			this._interface().qtMCIdle(movieController);
		}
	}

	/**
	 * Has movie controller or not?
	 * 
	 * @return boolean
	 */
	public boolean hasMovieController() {
		if (movieController == 0) {
			this.recoverMovieController();
		}

		return movieController != 0;
	}

	/**
	 * movie has changed.
	 */
	public void movieChanged() {
		if (this.hasMovieController() == true) {
			this._interface().qtMCMovieChanged(movieController, movie);
			this.doMovieTask();
		}
	}

	/**
	 * release the movie controller.
	 */
	public void releaseMovie() {
		if (movieController != 0) {
			this._interface().qtDisposeMovieController(movieController);
			movieController = 0;
		}

		super.releaseMovie();
	}

	/**
	 * start the movie.
	 */
	public void start() {
		if (this.hasMovieController() == true) {
			this.movieChanged();
			this._interface().qtMCDoAction(movieController,
					this._interface().mcActionPrerollAndPlay,
					this._interface().fixed1);
		}
	}

	/**
	 * recover the movie controller.
	 */
	protected void recoverMovieController() {
		if (this.hasMovie() == true) {
			Rectangle box = this.movieBox();
			movieController = this._interface().qtNewMovieController(
					movie, box.x, box.y, box.width, box.height,
					this._interface().mcTopLeftMovie +
							this._interface().mcNotVisible);
		}
	}
}
