package sra.jun.goodies.movie;

import java.awt.Component;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.Window;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import javax.swing.JPopupMenu;
import javax.swing.JRootPane;
import javax.swing.SwingUtilities;
import sra.smalltalk.StMenuViewForSwing;
import sra.smalltalk.StModel;
import sra.smalltalk.StView;
import sra.smalltalk.StViewJPanel;
import sra.smalltalk.WindowSpecLayout;
import sra.qt4jun.JunQTInterface;
import sra.jun.goodies.button.JunButtonViewSwing;
import sra.jun.goodies.track.JunTrackSliderViewSwing;

/**
 * JunMovieViewSwing class
 * 
 *  @author    NISHIHARA Satoshi
 *  @created   2001/01/24 (by NISHIHARA Satoshi)
 *  @updated   2002/11/11 (by nisinaka)
 *  @updated   2003/03/24 (by nisinaka)
 *  @version   445 (with StPL6.5) based on Jun490 for Smalltalk
 *  @copyright 1999-2004 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2004 Information-technology Promotion Agency, Japan (IPA)
 *  @copyright 2001-2004 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunMovieViewSwing.java,v 6.11 2004/04/15 06:27:59 nisinaka Exp $
 */
public class JunMovieViewSwing extends StViewJPanel implements JunMovieView {

	protected JunTrackSliderViewSwing sliderView;
	protected JunTrackSliderViewSwing sliderView2;
	protected JunButtonViewSwing firstButtonView;
	protected JunButtonViewSwing lastButtonView;
	protected JunButtonViewSwing previousButtonView;
	protected JunButtonViewSwing nextButtonView;
	protected JunButtonViewSwing loopButtonView;
	protected JunButtonViewSwing playButtonView;
	protected JunButtonViewSwing firstMarkerButtonView;
	protected JunButtonViewSwing lastMarkerButtonView;
	protected JPopupMenu _popupMenu;

	protected boolean inited = false;

	/**
	 * Create a new instance of JunMovieViewSwing and initialize it.
	 *
	 * @param aMoviePlayer sra.jun.goodies.movie.JunMoviePlayer
	 * @category Instance creation
	 */
	public JunMovieViewSwing(JunMoviePlayer aMoviePlayer) {
		super(aMoviePlayer);
	}

	/**
	 * Create a new view for the JunMoviePlayer.
	 * The view consists of a JunMovieView and two trackers.
	 *
	 * @param aMoviePlayer sra.jun.goodies.movie.JunMoviePlayer
	 * @return sra.smalltalk.StView
	 * @category Utilities
	 */
	public static StView WithDoubleTracker(JunMoviePlayer aMoviePlayer) {
		return new StViewJPanel(aMoviePlayer) {

			protected JunMovieViewSwing movieView;

			protected JunMovieViewSwing movieView() {
				if (movieView == null) {
					movieView = new JunMovieViewSwing((JunMoviePlayer) this.model());
				}
				return movieView;
			}

			protected void buildComponent() {
				this.setLayout(new WindowSpecLayout());
				this.add(this.movieView(), WindowSpecLayout.Position(0f, 1, 0f, 1, 1f, -1, 1f, -39));
				this.add(this.movieView().sliderView(), WindowSpecLayout.Position(0f, 77, 1f, -19, 1f, -77, 1f, -1));
				this.add(this.movieView().sliderView2(), WindowSpecLayout.Position(0f, 1, 1f, -38, 1f, -1, 1f, -20));
				this.add(this.movieView().firstButtonView(), WindowSpecLayout.Position(0f, 1, 1f, -19, 0f, 19, 1f, -1));
				this.add(this.movieView().lastButtonView(), WindowSpecLayout.Position(0f, 20, 1f, -19, 0f, 38, 1f, -1));
				this.add(this.movieView().previousButtonView(), WindowSpecLayout.Position(0f, 39, 1f, -19, 0f, 57, 1f, -1));
				this.add(this.movieView().nextButtonView(), WindowSpecLayout.Position(0f, 58, 1f, -19, 0f, 76, 1f, -1));
				this.add(this.movieView().loopButtonView(), WindowSpecLayout.Position(1f, -76, 1f, -19, 1f, -58, 1f, -1));
				this.add(this.movieView().playButtonView(), WindowSpecLayout.Position(1f, -57, 1f, -19, 1f, -39, 1f, -1));
				this.add(this.movieView().firstMarkerButtonView(), WindowSpecLayout.Position(1f, -38, 1f, -19, 1f, -20, 1f, -1));
				this.add(this.movieView().lastMarkerButtonView(), WindowSpecLayout.Position(1f, -19, 1f, -19, 1f, -1, 1f, -1));
				this.setPreferredSize(new Dimension(260, 180));
			}

			public void _setupKeyboard(Window aWindow) {
				this.movieView()._setupKeyboard(aWindow);
			}
		};
	}

	/**
	 * Create a new view for the JunMoviePlayer.
	 * The view consists of a JunMovieView and no tracker.
	 *
	 * @param aMoviePlayer sra.jun.goodies.movie.JunMoviePlayer
	 * @return sra.smalltalk.StView
	 * @category Utilities
	 */
	public static StView WithoutTracker(JunMoviePlayer aMoviePlayer) {
		return new StViewJPanel(aMoviePlayer) {

			protected JunMovieViewSwing movieView;

			protected JunMovieViewSwing movieView() {
				if (movieView == null) {
					movieView = new JunMovieViewSwing((JunMoviePlayer) this.model());
				}
				return movieView;
			}

			protected void buildComponent() {
				this.setLayout(new WindowSpecLayout());
				this.add(this.movieView(), WindowSpecLayout.Position(0f, 1, 0f, 1, 1f, -1, 1f, -1));
				this.setPreferredSize(new Dimension(260, 180));
			}

			public void _setupKeyboard(Window aWindow) {
				this.movieView()._setupKeyboard(aWindow);
			}
		};
	}

	/**
	 * Create a new view for the JunMoviePlayer.
	 * The view consists of a JunMovieView and a tracker.
	 *
	 * @param aMoviePlayer sra.jun.goodies.movie.JunMoviePlayer
	 * @return sra.smalltalk.StView
	 * @category Utilities
	 */
	public static StView WithTracker(JunMoviePlayer aMoviePlayer) {
		return new StViewJPanel(aMoviePlayer) {

			protected JunMovieViewSwing movieView;

			protected JunMovieViewSwing movieView() {
				if (movieView == null) {
					movieView = new JunMovieViewSwing((JunMoviePlayer) this.model());
				}
				return movieView;
			}

			protected void buildComponent() {
				this.setLayout(new WindowSpecLayout());
				this.add(this.movieView(), WindowSpecLayout.Position(0f, 1, 0f, 1, 1f, -1, 1f, -20));
				this.add(this.movieView().sliderView(), WindowSpecLayout.Position(0f, 39, 1f, -19, 1f, -39, 1f, -1));
				this.add(this.movieView().previousButtonView(), WindowSpecLayout.Position(0f, 1, 1f, -19, 0f, 19, 1f, -1));
				this.add(this.movieView().nextButtonView(), WindowSpecLayout.Position(0f, 20, 1f, -19, 0f, 38, 1f, -1));
				this.add(this.movieView().loopButtonView(), WindowSpecLayout.Position(1f, -38, 1f, -19, 1f, -20, 1f, -1));
				this.add(this.movieView().playButtonView(), WindowSpecLayout.Position(1f, -19, 1f, -19, 1f, -1, 1f, -1));
				this.setPreferredSize(new Dimension(260, 180));
			}

			public void _setupKeyboard(Window aWindow) {
				this.movieView()._setupKeyboard(aWindow);
			}
		};
	}

	/**
	 * Answer my model as a JunMoviePlayer.
	 * 
	 * @return sra.jun.goodies.movie.JunMoviePlayer
	 * @category accessing
	 */
	public JunMoviePlayer getMoviePlayer() {
		return (JunMoviePlayer) this.model();
	}

	/**
	 * Answer the current movie handle.
	 * 
	 * @return sra.jun.goodies.JunMovieHandle
	 * @category accessing
	 */
	public JunMovieHandle movieHandle() {
		return this.getMoviePlayer().movieHandle();
	}

	/**
	 * Set the bounds of the view.
	 * Also needs to set the movie for the new bounds.
	 *
	 * @param x The new <i>x</i>-coordinate of this component.
	 * @param y The new <i>y</i>-coordinate of this component.
	 * @param width The new <code>width</code> of this component.
	 * @param height The new <code>height</code> of this component.
	 * @see java.awt.Component#setBounds(int, int, int, int)
	 * @category bounds accessing
	 */
	public void setBounds(int x, int y, int width, int height) {
		super.setBounds(x, y, width, height);
		this.setMovieView();
	}

	/**
	 * Answer my default model.
	 * 
	 * @return sra.smalltalk.StModel
	 * @category model accessing
	 */
	protected StModel defaultModel() {
		return new JunMoviePlayer();
	}

	/**
	 * Answer the popup menu.
	 *
	 * @return javax.swing.JPopupMenu
	 * @category menu accessing
	 */
	protected JPopupMenu popupMenu() {
		if (_popupMenu != null) {
			this.remove(_popupMenu);
		}

		_popupMenu = StMenuViewForSwing.CreateJPopupMenu(this.getMoviePlayer().popupMenu());
		_popupMenu.setLightWeightPopupEnabled(false);
		this.add(_popupMenu);
		return _popupMenu;
	}

	/**
	 * Answer true if the view has a movie, otherwise false.
	 * 
	 * @return boolean
	 * @category testing
	 */
	public boolean hasMovie() {
		return this.getMoviePlayer().hasMovie();
	}

	/**
	 * Ansewr true if the view has a tracker, otherwise false.
	 * 
	 * @return boolean
	 * @category testing
	 */
	public boolean hasTracker() {
		Object[] dependents = this.getMoviePlayer().trackerModel().dependents();
		for (int i = 0; i < dependents.length; i++) {
			if (dependents[i] instanceof JunTrackSliderViewSwing) {
				return true;
			}
		}

		return false;
	}

	/**
	 * Ansewr true if the view has a second tracker, otherwise false.
	 * 
	 * @return boolean
	 * @category testing
	 */
	public boolean hasTracker2() {
		Object[] dependents = this.getMoviePlayer().trackerModel2().dependents();
		for (int i = 0; i < dependents.length; i++) {
			if (dependents[i] instanceof JunTrackSliderViewSwing) {
				return true;
			}
		}

		return false;
	}

	/**
	 * Display the movie.
	 * 
	 * @param aGraphics java.awt.Graphics
	 * @category displaying
	 */
	public void displayOn_(Graphics aGraphics) {
		if (inited == false) {
			inited = true;
			setMovieView();
		}
		SwingUtilities.invokeLater(new Runnable() {
			public void run() {
				if (isShowing() && hasMovie()) {
					movieHandle().updateMovie();
					movieHandle().doMovieTask();
				}
			}
		});
	}

	/**
	 * Build this component.
	 * 
	 * @category user interface
	 */
	protected void buildComponent() {
		this.setPreferredSize(new Dimension(240, 180));
	}

	/**
	 * Answer my first button view.
	 *
	 * @return sra.jun.goodies.button.JunButtonViewSwing
	 * @category user interface
	 */
	public JunButtonViewSwing firstButtonView() {
		if (firstButtonView == null) {
			firstButtonView = new JunButtonViewSwing(this.getMoviePlayer().trackerModel().firstButton());
		}
		return firstButtonView;
	}

	/**
	 * Answer my first marker button view.
	 *
	 * @return sra.jun.goodies.button.JunButtonViewSwing
	 * @category user interface
	 */
	public JunButtonViewSwing firstMarkerButtonView() {
		if (firstMarkerButtonView == null) {
			firstMarkerButtonView = new JunButtonViewSwing(this.getMoviePlayer().trackerModel().firstMarkerButton());
		}
		return firstMarkerButtonView;
	}

	/**
	 * Answer my last button view.
	 *
	 * @return sra.jun.goodies.button.JunButtonViewSwing
	 * @category user interface
	 */
	public JunButtonViewSwing lastButtonView() {
		if (lastButtonView == null) {
			lastButtonView = new JunButtonViewSwing(this.getMoviePlayer().trackerModel().lastButton());
		}
		return lastButtonView;
	}

	/**
	 * Answer my last marker button view.
	 *
	 * @return sra.jun.goodies.button.JunButtonViewSwing
	 * @category user interface
	 */
	public JunButtonViewSwing lastMarkerButtonView() {
		if (lastMarkerButtonView == null) {
			lastMarkerButtonView = new JunButtonViewSwing(this.getMoviePlayer().trackerModel().lastMarkerButton());
		}
		return lastMarkerButtonView;
	}

	/**
	 * Answer a loop button view.
	 *
	 * @return sra.jun.goodies.button.JunButtonViewSwing
	 * @category user interface
	 */
	public JunButtonViewSwing loopButtonView() {
		if (loopButtonView == null) {
			loopButtonView = new JunButtonViewSwing(this.getMoviePlayer().trackerModel().loopButton());
		}
		return loopButtonView;
	}

	/**
	 * Answer a next button view.
	 *
	 * @return sra.jun.goodies.button.JunButtonViewSwing
	 * @category user interface
	 */
	public JunButtonViewSwing nextButtonView() {
		if (nextButtonView == null) {
			nextButtonView = new JunButtonViewSwing(this.getMoviePlayer().trackerModel().nextButton());
		}
		return nextButtonView;
	}

	/**
	 * Answer my play button view.
	 *
	 * @return sra.jun.goodies.button.JunButtonViewSwing
	 * @category user interface
	 */
	public JunButtonViewSwing playButtonView() {
		if (playButtonView == null) {
			playButtonView = new JunButtonViewSwing(this.getMoviePlayer().trackerModel().playButton());
		}
		return playButtonView;
	}

	/**
	 * Answer my previous button view.
	 *
	 * @return sra.jun.goodies.button.JunButtonViewSwing
	 * @category user interface
	 */
	public JunButtonViewSwing previousButtonView() {
		if (previousButtonView == null) {
			previousButtonView = new JunButtonViewSwing(this.getMoviePlayer().trackerModel().previousButton());
		}
		return previousButtonView;
	}

	/**
	 * Answer my first slider view.
	 *
	 * @return sra.jun.goodies.track.JunTrackSliderViewSwing
	 * @category user interface
	 */
	public JunTrackSliderViewSwing sliderView() {
		if (sliderView == null) {
			sliderView = new JunTrackSliderViewSwing(this.getMoviePlayer().trackerModel());
		}
		return sliderView;
	}

	/**
	 * Answer my second slider view.
	 *
	 * @return sra.jun.goodies.track.JunTrackSliderViewSwing
	 * @category user interface
	 */
	public JunTrackSliderViewSwing sliderView2() {
		if (sliderView2 == null) {
			sliderView2 = new JunTrackSliderViewSwing(this.getMoviePlayer().trackerModel2());
		}
		return sliderView2;
	}

	/**
	 * Set up the keyboard for the view on the window.
	 * 
	 * @param aWindow java.awt.Window
	 * @see sra.smalltalk.StView#_setupKeyboard(java.awt.Window)
	 * @category keyboard
	 */
	public void _setupKeyboard(final Window aWindow) {
		this.addKeyListener(this.getMoviePlayer()._keyListener());
		this.firstButtonView().addKeyListener(this.getMoviePlayer()._keyListener());
		this.firstMarkerButtonView().addKeyListener(this.getMoviePlayer()._keyListener());
		this.lastButtonView().addKeyListener(this.getMoviePlayer()._keyListener());
		this.lastMarkerButtonView().addKeyListener(this.getMoviePlayer()._keyListener());
		this.loopButtonView().addKeyListener(this.getMoviePlayer()._keyListener());
		this.nextButtonView().addKeyListener(this.getMoviePlayer()._keyListener());
		this.playButtonView().addKeyListener(this.getMoviePlayer()._keyListener());
		this.previousButtonView().addKeyListener(this.getMoviePlayer()._keyListener());
		this.sliderView().addKeyListener(this.getMoviePlayer()._keyListener());
		this.sliderView2().addKeyListener(this.getMoviePlayer()._keyListener());

		this.addMouseListener(new MouseAdapter() {
			public void mouseEntered(MouseEvent e) {
				Component focusOwner = aWindow.getFocusOwner();
				if (focusOwner != null && focusOwner != JunMovieViewSwing.this) {
					JunMovieViewSwing.this.requestFocus();
				}
			}
		});

		aWindow.addWindowListener(new WindowAdapter() {
			public void windowActivated(WindowEvent e) {
				Component focusOwner = aWindow.getFocusOwner();
				if (focusOwner == null || focusOwner instanceof Container) {
					JunMovieViewSwing.this.requestFocus();
				}
			}
		});
	}

	/**
	 * Set the movie view.
	 * 
	 * @category private
	 */
	public void setMovieView() {
		if (this.topComponent() == null) {
			return;
		}

		if (this.hasMovie()) {
			int grafPtr = this.getMoviePlayer().grafPtr(inited);
			if (grafPtr != 0) {
				this.movieHandle().movieGrafPtr_(grafPtr);
				Rectangle box = this.movieBox();
				Point offset = JunQTInterface.Interface().getMovieOffsetSwing(this);
				box.translate(offset.x, offset.y);
				this.movieHandle().movieBox_(box);
				this.movieHandle().active_(true);
			}
		}
	}

	/**
	 * Answer the current area of the movie.
	 *
	 * @return java.awt.Rectangle
	 * @category private
	 */
	protected Rectangle movieBox() {
		if (this.getMoviePlayer().keepAspect() && this.hasMovie()) {
			Rectangle movieOriginalBox = new Rectangle(new Point(0, 0), this.movieHandle().originalExtent());
			Rectangle movieViewBox = new Rectangle(this._getLocationOnWindow(), this.getSize());
			return this.getMoviePlayer().AlignRectangle_withRectangle_(movieOriginalBox, movieViewBox);
		}

		return new Rectangle(this._getLocationOnWindow(), this.getSize());
	}

	/**
	 * Answer my location on the window.
	 *
	 * @return java.awt.Point
	 * @category private
	 */
	protected Point _getLocationOnWindow() {
		Point aPoint = new Point(0, 0);
		Component aComponent = this;
		while (aComponent instanceof JRootPane == false) {
			Point location = aComponent.getLocation();
			aPoint.translate(location.x, location.y);
			aComponent = aComponent.getParent();
		}
		return aPoint;
	}

}
