package sra.jun.goodies.multimedia;

import java.awt.Window;
import sra.smalltalk.StViewPanel;
import sra.smalltalk.WindowSpecLayout;
import sra.jun.goodies.files.JunFileViewAwt;

/**
 * JunMultiMediaViewerViewAwt class
 * 
 *  @author    nisinaka
 *  @created   2003/03/18 (by nisinaka)
 *  @updated   N/A
 *  @version   445 (with StPL6.5) based on Jun486 for Smalltalk
 *  @copyright 1999-2004 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2004 Information-technology Promotion Agency, Japan (IPA)
 *  @copyright 2001-2004 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunMultiMediaViewerViewAwt.java,v 6.7 2004/04/15 06:28:00 nisinaka Exp $
 */
public class JunMultiMediaViewerViewAwt extends StViewPanel implements JunMultiMediaViewerView {

	protected JunFileViewAwt fileView;
	protected JunMultiMediaImageViewAwt imageView;
	protected JunMultiMediaMovieViewAwt movieView;
	protected JunMultiMediaTextViewAwt textView;

	/**
	 * Create a new instance of JunMultiMediaViewerViewAwt and initialize it.
	 *
	 * @param aMultiMediaViewer sra.jun.goodies.multimedia.JunMultiMediaViewer
	 * @category Instance creation
	 */
	public JunMultiMediaViewerViewAwt(JunMultiMediaViewer aMultiMediaViewer) {
		super(aMultiMediaViewer);
	}

	/**
	 * Answer my model as JunMultiMediaViewer.
	 *
	 * @return sra.jun.goodies.multimedia.JunMultiMediaViewer
	 * @category model accessing
	 */
	public JunMultiMediaViewer getMultiMediaViewer() {
		return (JunMultiMediaViewer) this.model();
	}

	/**
	 * Answer my file view.
	 *
	 * @return sra.jun.goodies.files.JunFileViewAwt
	 * @category user interface
	 */
	public JunFileViewAwt fileView() {
		if (fileView == null) {
			fileView = new JunFileViewAwt(this.getMultiMediaViewer().fileModel());
		}
		return fileView;
	}

	/**
	 * Answer my image view.
	 *
	 * @return sra.jun.goodies.multimedia.JunMultiMediaImageViewAwt
	 * @category user interface
	 */
	public JunMultiMediaImageViewAwt imageView() {
		if (imageView == null) {
			imageView = new JunMultiMediaImageViewAwt(this.getMultiMediaViewer().imageModel());
			imageView.setVisible(false);
		}
		return imageView;
	}

	/**
	 * Answer my movie view.
	 *
	 * @return sra.jun.goodies.multimedia.JunMultiMediaMovieViewAwt
	 * @category user interface
	 */
	public JunMultiMediaMovieViewAwt movieView() {
		if (movieView == null) {
			movieView = new JunMultiMediaMovieViewAwt(this.getMultiMediaViewer().movieModel());
			movieView.setVisible(false);
		}
		return movieView;
	}

	/**
	 * Answer my text view.
	 *
	 * @return sra.jun.goodies.multimedia.JunMultiMediaTextViewAwt
	 * @category user interface
	 */
	public JunMultiMediaTextViewAwt textView() {
		if (textView == null) {
			textView = new JunMultiMediaTextViewAwt(this.getMultiMediaViewer().textModel());
			textView.setVisible(false);
		}
		return textView;
	}

	/**
	 * Build this component.
	 * 
	 * @category user interface
	 */
	protected void buildComponent() {
		this.setLayout(new WindowSpecLayout());
		this.add(this.fileView(), WindowSpecLayout.Position(0f, 1, 0f, 1, 1f, -1, 0f, 290));
		this.add(this.imageView(), WindowSpecLayout.Position(0f, 8, 0f, 300, 1f, -8, 1f, -8));
		this.add(this.movieView(), WindowSpecLayout.Position(0f, 8, 0f, 300, 1f, -8, 1f, -8));
		this.add(this.textView(), WindowSpecLayout.Position(0f, 8, 0f, 300, 1f, -8, 1f, -8));
		this.setSize(480, 580);
	}

	/**
	 * Set up the keyboard for the view on the window.
	 * 
	 * @param aWindow java.awt.Window
	 * @see sra.smalltalk.StView#_setupKeyboard(java.awt.Window)
	 * @category keyboard
	 */
	public void _setupKeyboard(Window aWindow) {
		this.movieView().getMovieView()._setupKeyboard(aWindow);
	}

}
