package sra.jun.goodies.nib;

import java.awt.Color;
import sra.smalltalk.StBlockClosure;
import sra.smalltalk.StSymbol;
import sra.smalltalk.StView;
import sra.jun.goodies.colors.JunColorChoiceModel;
import sra.jun.goodies.colors.JunColorChoiceSBH;
import sra.jun.system.support.JunSystem;

/**
 * JunNibChoiceWithColor class
 * 
 *  @author    Nobuto Matsubara
 *  @created   2004/03/26 (By Nobuto Matsubara)
 *  @updated   N/A
 *  @version   445 (with StPL6.5) based on Jun538 for Smalltalk
 *  @copyright 1999-2004 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2004 Information-technology Promotion Agency, Japan (IPA)
 *  @copyright 2001-2004 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunNibChoiceWithColor.java,v 6.1 2004/04/15 06:28:00 nisinaka Exp $
 */
public class JunNibChoiceWithColor extends JunNibChoice {

	protected JunColorChoiceModel colorModel;

	/**
	 * Create a new instance of JunNibChoiceWithColor.
	 * 
	 * @category Instance creation
	 */
	public JunNibChoiceWithColor() {
		super();
	}

	/**
	 * Create an instance of JunNibChoiceWithColor.
	 * 
	 * @params valueAray Object[]
	 * @category Instance creation
	 */
	public JunNibChoiceWithColor(Object[] valueArray) {
		this.value_(valueArray);
	}

	/**
	 * Create an instance of JunNibChoice.
	 * 
	 * @params aCollection double[]
	 * @category Instance creation
	 */
	public JunNibChoiceWithColor(double[] aCollection) {
		this.nibCollection_(aCollection);
	}

	/**
	 * Create an instance of JunNibChoice.
	 * 
	 * @params aCollection double[]
	 * @params nibWidth double
	 * @params nibcolor java.awt.Color
	 * @params nibShape sra.smalltalk.StSymbol
	 * @category Instance creation
	 */
	public JunNibChoiceWithColor(double[] aCollection, double nibWidth, Color nibColor, StSymbol nibShape) {
		this.value_(new Object[] { aCollection, new Double(nibWidth), nibColor, nibShape });
	}

	/**
	 * Initializing.
	 * 
	 * @category initialize-release
	 */
	protected void initialize() {
		super.initialize();
		colorModel = null;
	}

	/**
	 * Answer the color model.
	 * 
	 * @return sra.jun.goodies.color.JunColorChoiceModel
	 * @category accessing
	 */
	public JunColorChoiceModel colorModel() {
		if (colorModel == null) {
			colorModel = new JunColorChoiceSBH();
			colorModel.color_(this.nibColor());
			colorModel.compute_(new StBlockClosure() {
				public Object value() {
					JunNibChoiceWithColor.this.nibColor_(colorModel.color());
					return Boolean.TRUE;
				}
			});
		}
		return colorModel;
	}

	/**
	 * Set my new nib color.
	 * 
	 * @param aColor java.awt.Color
	 * @see sra.jun.goodies.nib.JunNibChoice#nibColor_(java.awt.Color)
	 * @category accessing
	 */
	public void nibColor_(Color aColor) {
		if (this.colorModel().color().equals(aColor) == false) {
			this.colorModel().color_(aColor);
		}
		super.nibColor_(aColor);
	}

	/**
	 * Answer a default view.
	 * 
	 * @return sra.smalltalk.StView
	 * @category defaults
	 */
	public StView defaultView() {
		if (GetDefaultViewMode() == VIEW_AWT) {
			return new JunNibChoiceWithColorViewAwt(this);
		} else {
			return new JunNibChoiceWithColorViewSwing(this);
		}
	}

	/**
	 * Answer a window title.
	 * 
	 * @return java.lang.String
	 * @category interface opening
	 */
	protected String windowTitle() {
		return JunSystem.$String("Nib Choice");
	}

}
