package sra.jun.goodies.plotter;

import java.awt.*;
import java.util.*;

import sra.jun.geometry.basic.*;
import sra.jun.goodies.pen.*;
import sra.smalltalk.*;

/**
 * JunPlotterViewAwt class
 * 
 *  @author    Nobuto Matsubara
 *  @created   2003/10/31 (by Nobuto Matsubara)
 *  @updated   N/A
 *  @version   445 (with StPL6.5) based on JunXXX for Smalltalk
 *  @copyright 1999-2004 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2004 Information-technology Promotion Agency, Japan (IPA)
 *  @copyright 2001-2004 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunPlotterViewAwt.java,v 1.6 2004/04/15 06:28:01 nisinaka Exp $
 */
public class JunPlotterViewAwt extends StViewCanvas implements JunPlotterView {
	private JunPenTransformation viewingTransformation;
	protected PopupMenu _popupMenu;
	/**
	 * Create a new instance of JunPlotterViewAwt and initialize it.
	 *
	 * @param aModel sra.jun.goodies.plotter.JunPlotter
	 * @category instance creation
	 */
	public JunPlotterViewAwt(JunPlotter aModel) {
		super(aModel);
	}
	
	/**
	 * Answer a JunTransformation object.
	 * 
	 * @return sra.jun.goodies.pen.JunPenTransformation
	 * @category accessing
	 */
	public JunPenTransformation viewingTransformation() {
		if (viewingTransformation == null) {
			viewingTransformation = JunPenTransformation.Cavalie_(JunPenAngle.FromDeg_(45));
		}
		return viewingTransformation;
	}
	
	/**
	 * Set a JunTransformation object.
	 * 
	 * @param aTransformation sra.jun.goodies.pen.JunPenTransformation
	 * @category accessing
	 */
	public void viewingTransformation_(JunPenTransformation aTransformation) {
		viewingTransformation = aTransformation;
		this.repaint();
	}
	
	/**
	 * Answer origin point.
	 * 
	 * @return Point
	 * @category accessing
	 */
	public Jun2dPoint originPoint() {
		Jun2dPoint aPoint = new Jun2dPoint(this.getBounds().getCenterX(), this.getBounds().getCenterY());
		return aPoint;
	}
	
	/**
	 * Answer transformation.
	 * 
	 * @return sra.jun.goodies.pen.JunPenTransformation
	 * @category accessing
	 */
	public JunPenTransformation displayingTransformation() {
		JunPenTransformation aTransformation = this.viewingTransformation();
		aTransformation = aTransformation.product_(JunPenTransformation.Translate_(this.originPoint()));
		return aTransformation;
	}
	
	/**
	 * Clear.
	 * 
	 * @category displaying
	 */
	public void clear() {
		Graphics aGraphics = this.getGraphics();
		aGraphics.clearRect(this.getBounds().x, this.getBounds().y, this.getBounds().width, this.getBounds().height);
	}
	
	/**
	 * Display.
	 * 
 	 * @param graphicsContext Graphics
 	 * @category displaying
	 */
	public void displayOn_(final Graphics graphicsContext) {
		final JunPlotterViewAwt this_ = this;
		this.getModel().drawingsDo_(new StBlockClosure() {
			public Object value_(Object array) {
				this_.draw_on_transform_((Vector)array, graphicsContext, this_.displayingTransformation());
				return (Object)this;
			}
		});
	}
	
	/**
	 * Drawing an array on a graphics context.
	 * 
	 * @param anArray java.lang.Object[]
	 * @category displaying
	 */
	public void draw_(Vector anArray) {
		this.draw_on_(anArray, this.getGraphics());
	}
	
	/**
	 * Drawing an array on a graphics context.
	 * 
	 * @param anArray java.lang.Object[]
	 * @param graphicsContext Graphics
	 * @category displaying
	 */
	public void draw_on_(Vector anArray, Graphics graphicsContext) {
		this.draw_on_transform_(anArray, graphicsContext, this.displayingTransformation());
	}
	
	/**
	 * Drawing an array on a graphics context.
	 * 
	 * @param anArray java.lang.Object[]
	 * @param graphicsContext Graphics
	 * @param aTransformation sra.jun.goodies.pen.JunPenTransformation
	 * @category displaying
	 */
	public void draw_on_transform_(Vector anArray, Graphics graphicsContext, JunPenTransformation aTransformation) {
		StSymbol tagSymbol = (StSymbol) anArray.get(0);
		if (tagSymbol == $("stroke")) {
			int nibWidth = ((Integer)anArray.get(1)).intValue();
			Color colorValue = (Color)anArray.get(2);
			JunPenLocation fromPoint = (JunPenLocation)anArray.get(3);
			JunPenLocation toPoint = (JunPenLocation)anArray.get(4);
			Jun2dPoint fromPoint2d = (fromPoint.transform_(aTransformation)).asPoint().rounded();
			Jun2dPoint toPoint2d = (toPoint.transform_(aTransformation)).asPoint().rounded();
			StRectangle aRectangle = StRectangle.Vertex_vertex_(fromPoint2d._toPoint(), toPoint2d._toPoint());
			aRectangle = aRectangle.expandedBy_(nibWidth);
			if (aRectangle.intersects_(new StRectangle(graphicsContext.getClipBounds()))) {
				((Graphics2D)graphicsContext).setStroke(new BasicStroke(nibWidth));
				graphicsContext.setColor(colorValue);
				graphicsContext.drawLine((int)fromPoint2d.x(), (int)fromPoint2d.y(), (int)toPoint2d.x(), (int)toPoint2d.y());
			}
		}
		if (tagSymbol == $("fill")) {
			Color colorValue = (Color)anArray.get(1);
			Vector locationCollection = (Vector)anArray.get(2);
			Jun2dBoundingBox aRectangle = null;
			Polygon aPolygon = new Polygon();
			for (ListIterator aLocation = locationCollection.listIterator(); aLocation.hasNext();) {
				Jun2dPoint aPoint = ((JunPenLocation)aLocation.next()).transform_(aTransformation).asPoint().rounded();
				aPolygon.addPoint((int)aPoint.x(), (int)aPoint.y());
				if (aRectangle == null) {
					aRectangle = aPoint.extent_(new Jun2dPoint(1.0, 1.0));
				} else {
					aRectangle = aRectangle.merge_(aPoint.extent_(new Jun2dPoint(1.0, 1.0)));
				}
			}
			if (aRectangle.intersects_(new Jun2dBoundingBox(graphicsContext.getClipBounds()))) {
				graphicsContext.setColor(colorValue);
				graphicsContext.fillPolygon(aPolygon);
			}
		}
	}
	
	/**
	 * Updating.
	 * 
	 * @param aspect sra.smalltalk.StSymbol
	 * @param parameter java.lang.Object[]
	 */
	public void update_with_(StSymbol aspect, Vector parameter) {
		if (aspect == $("clear")) {
			this.clear();
		}
		if (aspect == $("draw")) {
			this.draw_(parameter);
		}
		this.repaint();
	}

	/**
	 * Build this component.
	 * 
	 * @category interface opening
	 */
	protected void buildComponent() {
		this.setSize(300, 300);
	}

	/**
	 * Answer my default controller. Subclasses may override this.
	 * 
	 * @return sra.smalltalk.StController
	 * @category controller accessing
	 */
	protected StController defaultController() {
		return new JunPlotterController();
	}
	
	/**
	 * Answer a model.
	 * 
	 * @return sra.jun.goodies.JunPlotter
	 * @category accessing
	 */
	public JunPlotter getModel() {
		return (JunPlotter)this.model();
	}

	/**
	 * Answer the popup menu.
	 *
	 * @return java.awt.PopupMenu
	 */
	protected PopupMenu popupMenu() {
		if (_popupMenu == null) {
			_popupMenu = this._createPopupMenu(this.getModel().popupMenu());
			this.add(_popupMenu);
		}
		return _popupMenu;
	}
	
	/**
	 * Answer a JunPlotterController object.
	 * 
	 * @return sra.jun.goodies.JunPlotterController
	 * @category controller accessing
	 */
	public JunPlotterController plotterController() {
		return (JunPlotterController)this.controller();
	}
}
