package sra.jun.goodies.utilities;

import java.util.*;
import sra.smalltalk.*;

/**
 * JunStringUtility class
 * 
 *  @author    Mitsuhiro Asada
 *  @created   2003/05/27 (by Mitsuhiro Asada)
 *  @updated   N/A
 *  @version   445 (with StPL6.5) based on Jun500 for Smalltalk
 *  @copyright 1999-2004 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2004 Information-technology Promotion Agency, Japan (IPA)
 *  @copyright 2001-2004 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunStringUtility.java,v 6.8 2004/04/15 06:28:03 nisinaka Exp $
 */
public class JunStringUtility {

	/**
	 * Answer separated strings.
	 *
	 * @return java.lang.String[]
	 * @param separateCollection java.lang.String
	 * @param dividerCollection char[]
	 * @category Separating
	 */
	public static String[] Separate_dividers_(String separateCollection, char[] dividerCollection) {
		Vector indexCollection = new Vector();
		indexCollection.add(new Integer(-1));
		for (int index = 0; index < separateCollection.length(); index++) {
			char each = separateCollection.charAt(index);
			for (int j = 0; j < dividerCollection.length; j++) {
				if (dividerCollection[j] == each) {
					indexCollection.add(new Integer(index));
				}
			}
		}
		indexCollection.add(new Integer(separateCollection.length()));

		Vector intervalCollection = new Vector();
		for (int i = 0; i < indexCollection.size() - 1; i++) {
			int s = ((Integer) indexCollection.get(i)).intValue();
			int e = ((Integer) indexCollection.get(i + 1)).intValue();
			intervalCollection.add(new StInterval(s + 1, e - 1));
		}

		Vector result = new Vector();
		for (int i = 0; i < intervalCollection.size(); i++) {
			StInterval interval = (StInterval) intervalCollection.get(i);
			if (interval.size() > 0) {
				String part = separateCollection.substring((int) interval.first(), (int) interval.last() + 1);
				result.add(part);
			}
		}

		String[] strings = new String[result.size()];
		result.copyInto(strings);
		return strings;
	}

	/**
	 * Answer the separator charactors.
	 *
	 * @return char[]
	 * @category Separating
	 */
	public static char[] Separators() {
		char[] separators = new char[5];
		separators[0] = ' ';
		separators[1] = '\t';
		separators[2] = '\r';
		separators[3] = '\f';
		separators[4] = '\n';
		return separators;
	}

	/**
	 * Shorten by ellipsis if too long.
	 *
	 * @return java.lang.String
	 * @param aString java.lang.String
	 * @param charCount int
	 * @category Converting
	 */
	public static String _ContractTo_(String aString, int charCount) {
		String shortString = (aString == null) ? new String("") : new String(aString);
		if (aString.length() > charCount) {
			int half = charCount / 2;
			shortString = aString.substring(0, half - 1) + "..." + aString.substring(aString.length() - (charCount - half) + 2);
		}
		return shortString;
	}

	/**
	 * Answer true if the string matches the pattern, otherwise false.
	 * 
	 * @param string java.lang.String
	 * @param pattern java.lang.String
	 * @return boolean
	 * @category Comparing
	 */
	public static boolean StringMatch_and_(String string, String pattern) {
		char[] stringCharacters = new char[string.length() + 1];
		string.getChars(0, string.length(), stringCharacters, 0);
		stringCharacters[string.length()] = 0;
		char[] patternCharacters = new char[pattern.length() + 1];
		pattern.getChars(0, pattern.length(), patternCharacters, 0);
		patternCharacters[pattern.length()] = 0;
		return StringMatch_sIndex_and_pIndex_(stringCharacters, 0, patternCharacters, 0);
	}

	/**
	 * Answer true if the string matches the pattern, otherwise false.
	 * 
	 * @param string java.lang.String
	 * @param sindex int
	 * @param pattern java.lang.String
	 * @param pindex int
	 * @return boolean
	 * @cagtegory Comparing
	 */
	public static boolean StringMatch_sIndex_and_pIndex_(char[] string, int sindex, char[] pattern, int pindex) {
		int si = sindex;
		int pi = pindex;
		char scc = string[si];
		si++;
		char c = pattern[pi];
		pi++;
		if (c == '[') {
			boolean ok = false;
			char lc = 255;

			while ((c = pattern[pi++]) != 0) {
				if (c == ']') {
					if (ok) {
						return StringMatch_sIndex_and_pIndex_(string, si, pattern, pi);
					} else {
						return false;
					}
				} else {
					if (c == '-') {
						if (lc <= scc && scc <= pattern[pi]) {
							ok = true;
						}
						pi++;
					} else {
						lc = c;
						if (scc == c) {
							ok = true;
						}
					}
				}
			}
		}
		if (c == '?') {
			if (scc != 0) {
				return StringMatch_sIndex_and_pIndex_(string, si, pattern, pi);
			} else {
				return false;
			}
		}
		if (c == '*') {
			if (pattern[pi] == 0) {
				return true;
			}
			si--;
			while (string[si] != 0) {
				if (StringMatch_sIndex_and_pIndex_(string, si, pattern, pi)) {
					return true;
				}
				si++;
			}
			return false;
		}
		if (c == 0) {
			return scc == 0;
		}
		if (c != scc) {
			return false;
		}
		if (scc != 0) {
			return StringMatch_sIndex_and_pIndex_(string, si, pattern, pi);
		} else {
			return false;
		}
	}

}
