package sra.jun.opengl.display;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Frame;
import java.awt.Image;
import java.awt.Rectangle;
import java.util.Hashtable;
import java.util.Vector;
import sra.smalltalk.StBlockClosure;
import sra.smalltalk.StButtonGroup;
import sra.smalltalk.StCheckBoxMenuItem;
import sra.smalltalk.StMenu;
import sra.smalltalk.StMenuItem;
import sra.smalltalk.StMenuModel;
import sra.smalltalk.StRadioButtonMenuItem;
import sra.smalltalk.StSymbol;
import sra.smalltalk.StValueHolder;
import sra.smalltalk.StView;
import sra.gl4jun.GLjImageSource;
import sra.jun.geometry.basic.Jun2dPoint;
import sra.jun.geometry.basic.Jun3dBoundingBox;
import sra.jun.geometry.basic.Jun3dPoint;
import sra.jun.geometry.basic.JunAngle;
import sra.jun.geometry.curves.Jun3dLine;
import sra.jun.geometry.surfaces.JunPlane;
import sra.jun.geometry.surfaces.JunSphereSurface;
import sra.jun.geometry.transformations.Jun3dTransformation;
import sra.jun.goodies.progress.JunCursorAnimator;
import sra.jun.opengl.lights.JunOpenGLLight;
import sra.jun.opengl.objects.JunOpenGL3dCompoundObject;
import sra.jun.opengl.objects.JunOpenGL3dObject;
import sra.jun.opengl.projection.JunOpenGLParallelProjection;
import sra.jun.opengl.projection.JunOpenGLPerspective;
import sra.jun.opengl.projection.JunOpenGLProjection;
import sra.jun.opengl.projection.JunOpenGLProjector;
import sra.jun.opengl.support.JunOpenGLRenderingContext;
import sra.jun.opengl.texture.JunOpenGLTexture;
import sra.jun.system.framework.JunApplicationModel;
import sra.jun.system.framework.JunDialog;
import sra.jun.system.support.JunSmallCompiler;
import sra.jun.system.support.JunSystem;

/**
 * JunOpenGL3dModel class
 * 
 *  @author    MATSUDA Ryouichi
 *  @created   2000/01/06 (by MATSUDA Ryouichi)
 *  @updated   2003/05/13 (by nisinaka)
 *  @updated   2004/03/26 (by nisinaka)
 *  @version   445 (with StPL6.5) based on Jun493 for Smalltalk
 *  @copyright 1999-2004 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2004 Information-technology Promotion Agency, Japan (IPA)
 *  @copyright 2001-2004 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunOpenGL3dModel.java,v 6.12 2004/04/15 06:28:07 nisinaka Exp $
 */
public class JunOpenGL3dModel extends JunApplicationModel {

	public static final Dimension DefaultImageExtent = new Dimension(256, 256);
	protected static final double ThetaAccuracy = 0.001;

	protected JunOpenGL3dObject openGL3dObject;
	protected JunOpenGLProjector openGLProjector;
	protected JunOpenGLDisplayLight[] displayLights;
	protected Vector selectedObjects;
	protected Hashtable defaultProjectionTable;
	protected StMenuModel _menuModel;
	protected StMenu _viewMenu;
	protected StMenu _lightMenu;

	/**
	 * Create a new instance of JunOpenGL3dModel and initialize it.
	 * 
	 * @category Instance creation
	 */
	public JunOpenGL3dModel() {
		super();
	}

	/**
	 * Create a new instance of JunOpenGL3dModel and initialize it with the JunOpenGL3dObject.
	 * 
	 * @param a3dObject sra.jun.opengl.objects.JunOpenGL3dObject
	 * @category Instance creation
	 */
	public JunOpenGL3dModel(JunOpenGL3dObject a3dObject) {
		super();
		this.openGL3dObject_(a3dObject);
	}

	/**
	 * Initialize the receiver.
	 * 
	 * @category initialize-release
	 */
	protected void initialize() {
		super.initialize();

		openGL3dObject = null;
		openGLProjector = null;
		displayLights = null;
		selectedObjects = null;
		defaultProjectionTable = null;
		_menuModel = null;
		_viewMenu = null;
		_lightMenu = null;
	}

	/**
	 * Answer my current display object.
	 * 
	 * @return sra.jun.opengl.objects.JunOpenGL3dObject
	 * @category accessing
	 */
	public JunOpenGL3dObject openGL3dObject() {
		return openGL3dObject;
	}

	/**
	 * Set my new display object.
	 * 
	 * @param a3dObject sra.jun.opengl.objects.JunOpenGL3dObject
	 * @category accessing
	 */
	public void openGL3dObject_(JunOpenGL3dObject a3dObject) {
		openGL3dObject = a3dObject;
		this.clearSelectedObjects();
		this.updateMenuIndication();
	}

	/**
	 * Answer my current projector.
	 * 
	 * @return sra.jun.opengl.projection.JunOpenGLProjector
	 * @category accessing
	 */
	public JunOpenGLProjector openGLProjector() {
		if (openGLProjector == null) {
			openGLProjector = new JunOpenGLProjector();
			openGLProjector.eyePoint_(this.defaultEyePoint());
			openGLProjector.sightPoint_(this.defaultSightPoint());
			openGLProjector.upVector_(this.defaultUpVector());
			openGLProjector.viewFactor_(this.defaultViewFactor());
			openGLProjector.zoomHeight_(this.defaultZoomHeight());
			openGLProjector.presentation_(this.defaultPresentation());
			if (this.defaultProjection() == $("parallelProjection")) {
				openGLProjector.parallelProjection();
			} else {
				openGLProjector.perspectiveProjection();
			}
		}

		return openGLProjector;
	}

	/**
	 * Answer my current projection.
	 * 
	 * @return sra.jun.opengl.projection.JunOpenGLProjection
	 * @category accessing
	 */
	public JunOpenGLProjection openGLProjection() {
		return this.openGLProjector().projection();
	}

	/**
	 * Set my new projection.
	 * 
	 * @param aProjection sra.jun.opengl.projection.JunOpenGLProjection
	 * @category accessing
	 */
	public void openGLProjection_(JunOpenGLProjection aProjection) {
		this.openGLProjector().projection_(aProjection);
	}

	/**
	 * Flush the object.
	 * 
	 * @category flushing
	 */
	public void flushObject() {
		openGL3dObject = null;
	}

	/**
	 * Flush the projector.
	 * 
	 * @category flushing
	 */
	public void flushProjector() {
		openGLProjector = null;
	}

	/**
	 * Answer the bounding box of the display object.
	 * 
	 * @return sra.jun.geometry.basic.Jun3dBoundingBox
	 * @category bounds accessing
	 */
	public Jun3dBoundingBox boundingBox() {
		if (this.openGL3dObject() == null) {
			return Jun3dBoundingBox.Origin_corner_(new Jun3dPoint(0, 0, 0), new Jun3dPoint(0, 0, 0));
		}
		return this.openGL3dObject().boundingBox();
	}

	/**
	 * Answer whether the display object is empty or not.
	 *
	 * @return boolean
	 * @category testing
	 */
	public boolean isEmpty() {
		if (this.openGL3dObject() == null) {
			return true;
		}

		if (this.openGL3dObject.isCompound() == false) {
			return false;
		}

		return ((JunOpenGL3dCompoundObject) this.openGL3dObject()).components().length == 0;
	}

	/**
	 * Enumerate all points of the 3D object and evaluate the block.
	 * 
	 * @param aBlock sra.smalltalk.StBlockClosure
	 * @category enumerating
	 */
	public void pointsDo_(StBlockClosure aBlock) {
		if (this.openGL3dObject() == null) {
			return;
		}

		this.openGL3dObject().pointsDo_(aBlock);
	}

	/**
	 * Convert the view as an image.
	 *
	 * @return java.awt.Image
	 * @category converting
	 */
	public Image asImage() {
		return this.asImage(this.defaultImageExtent());
	}

	/**
	 * Convert the view as an image.
	 *
	 * @param extent java.awt.Dimension
	 * @return java.awt.Image
	 * @category converting
	 */
	public synchronized Image asImage(Dimension extent) {
		StView aView = this.getView();
		Color backgroundColor = (aView == null) ? Color.white : aView.toComponent().getBackground();
		GLjImageSource imageSource = new GLjImageSource(extent.width, extent.height);
		JunOpenGLRenderingContext renderingContext = null;
		try {
			renderingContext = new JunOpenGLRenderingContext(imageSource);
			renderingContext.gljInitialize();
			renderingContext.gljCurrent();
			this.renderOn_(renderingContext);
		} finally {
			if (renderingContext != null) {
				renderingContext.gljFlush();
				renderingContext.gljRelease();
				renderingContext = null;
			}
		}

		return imageSource.getImage();
	}

	/**
	 * Answer the first display light.
	 * 
	 * @return sra.jun.opengl.display.JunOpenGLDisplayLight
	 * @category lighting
	 */
	public JunOpenGLDisplayLight displayLight1() {
		return this.displayLights()[0];
	}

	/**
	 * Answer the second display light.
	 * 
	 * @return sra.jun.opengl.display.JunOpenGLDisplayLight
	 * @category lighting
	 */
	public JunOpenGLDisplayLight displayLight2() {
		return this.displayLights()[1];
	}

	/**
	 * Answer the third display light.
	 * 
	 * @return sra.jun.opengl.display.JunOpenGLDisplayLight
	 * @category lighting
	 */
	public JunOpenGLDisplayLight displayLight3() {
		return this.displayLights()[2];
	}

	/**
	 * Answer the fourth display light.
	 * 
	 * @return sra.jun.opengl.display.JunOpenGLDisplayLight
	 * @category lighting
	 */
	public JunOpenGLDisplayLight displayLight4() {
		return this.displayLights()[3];
	}

	/**
	 * Answer the fifth display light.
	 * 
	 * @return sra.jun.opengl.display.JunOpenGLDisplayLight
	 * @category lighting
	 */
	public JunOpenGLDisplayLight displayLight5() {
		return this.displayLights()[4];
	}

	/**
	 * Answer the array of lights which are on.
	 * 
	 * @return sra.jun.opengl.lights.JunOpenGLLight[]
	 * @category lighting
	 */
	public synchronized JunOpenGLLight[] displayLightCollection() {
		Vector lightCollection = new Vector();
		Jun3dTransformation t = (Jun3dTransformation) this.openGLProjection().asEyeTransformation().inverse();
		for (int i = 0; i < this.displayLights().length; i++) {
			JunOpenGLDisplayLight displayLight = this.displayLights()[i];
			if (displayLight.isOn()) {
				lightCollection.addElement(displayLight.light().transform_(t));
			}
		}

		JunOpenGLLight[] lights = new JunOpenGLLight[lightCollection.size()];
		lightCollection.copyInto(lights);

		return lights;
	}

	/**
	 * Answer the array of all display lights.
	 * 
	 * @return sra.jun.opengl.display.JunOpenGLDisplayLight[]
	 * @category lighting
	 */
	public synchronized JunOpenGLDisplayLight[] displayLights() {
		if (displayLights == null) {
			displayLights = new JunOpenGLDisplayLight[5];
			displayLights[0] = JunOpenGLDisplayLight.ParallelLight_color_position_(true, this.defaultLightColor(), this.defaultLightPoint());
			displayLights[1] = new JunOpenGLDisplayLight();
			displayLights[2] = new JunOpenGLDisplayLight();
			displayLights[3] = new JunOpenGLDisplayLight();
			displayLights[4] = JunOpenGLDisplayLight.AmbientLight_color_(true, this.defaultLightColor());

			final JunOpenGL3dModel self = this;
			for (int i = 0; i < displayLights.length; i++) {
				displayLights[i].compute_(new StBlockClosure() {
					public Object value() {
						self.updateLightMenuIndication();
						self.changed_($("light"));
						return null;
					}
				});
			}

			String[] name = new String[] { JunSystem.$String("Light") + "1", JunSystem.$String("Light") + "2", JunSystem.$String("Light") + "3", JunSystem.$String("Light") + "4", JunSystem.$String("Ambient Light", "Ambient")};
			for (int i = 0; i < displayLights.length; i++) {
				displayLights[i].lightName_(name[i]);
			}
		}

		return displayLights;
	}

	/**
	 * Set my new display lights.
	 * 
	 * @param anArrayOfDisplayLight sra.jun.opengl.display.JunOpenGLDisplayLight[]
	 * @category lighting
	 */
	public synchronized void displayLights_(JunOpenGLDisplayLight[] anArrayOfDisplayLight) {
		if (anArrayOfDisplayLight == null || anArrayOfDisplayLight.length != 5) {
			return;
		}

		displayLights = anArrayOfDisplayLight;
	}

	/**
	 * Render the display objects on the redering context.
	 * 
	 * @param renderingContext sra.jun.opengl.objects.JunOpenGLRenderingContext
	 * @category rendering
	 */
	public void renderOn_(JunOpenGLRenderingContext renderingContext) {
		renderingContext.clear();
		JunOpenGLLight[] lightCollection = this.displayLightCollection();
		JunOpenGL3dObject renderingObject = this.openGL3dObject();
		if (renderingObject == null) {
			this.openGLProjector().projectOn_(renderingContext);
		} else {
			if (this.selectedObjects().isEmpty()) {
				this.openGLProjector().project_withLights_on_(renderingObject, lightCollection, renderingContext);
			} else {
				JunOpenGLProjector aProjector = (JunOpenGLProjector) this.openGLProjector().copy();
				if (this.openGLProjector().presentation() == $("solidPresentation")) {
					aProjector.wireframePresentation();
				} else if (this.openGLProjector().presentation() == $("wireframePresentation")) {
					aProjector.wireframePresentation();
				} else if (this.openGLProjector().presentation() == $("hiddenlinePresentation")) {
					aProjector.hiddenlinePresentation();
				}
				aProjector.project_withLights_on_(renderingObject, lightCollection, renderingContext);
				if (this.openGLProjector().presentation() == $("solidPresentation")) {
					aProjector.solidPresentation();
				} else if (this.openGLProjector().presentation() == $("wireframePresentation")) {
					aProjector.solidPresentation();
				} else if (this.openGLProjector().presentation() == $("hiddenlinePresentation")) {
					aProjector.solidPresentation();
				}
				Object[] objects = this.selectedObjects().toArray();
				for (int i = 0; i < objects.length; i++) {
					aProjector.project_on_((JunOpenGL3dObject) objects[i], renderingContext);
				}
			}
		}
	}

	/**
	 * Grab and rotate the 3d object.
	 * 
	 * @param deltaPoint sra.jun.grometry.basic.Jun2dPoint
	 * @category manipulating
	 */
	public void grab_(Jun2dPoint deltaPoint) {
		this.grab_(deltaPoint, false);
	}

	/**
	 * Grab and rotate the 3d object.
	 * 
	 * @param deltaPoint sra.jun.grometry.basic.Jun2dPoint
	 * @param shiftDown boolean
	 * @category manipulating
	 */
	public void grab_(Jun2dPoint deltaPoint, boolean shiftDown) {
		if (this.openGL3dObject() == null) {
			return;
		}

		Jun2dPoint fromPoint = new Jun2dPoint(0, 0);
		Jun2dPoint toPoint;

		if (shiftDown) {
			toPoint = new Jun2dPoint(deltaPoint.x() / 10, deltaPoint.y() / 10);
		} else {
			toPoint = new Jun2dPoint(deltaPoint.x(), deltaPoint.y());
		}

		this.grab_xy_(fromPoint, toPoint);
	}

	/**
	 * Grab and rotate the 3d object.
	 * 
	 * @param from2dPoint sra.jun.grometry.basic.Jun2dPoint
	 * @param to2dPoint sra.jun.grometry.basic.Jun2dPoint
	 * @category manipulating
	 */
	public void grab_xy_(Jun2dPoint from2dPoint, Jun2dPoint to2dPoint) {
		if (this.openGL3dObject() == null) {
			return;
		}

		JunOpenGLProjection aProjection = this.openGLProjection();
		JunSphereSurface grabSphere = this.grabSphere();
		Jun3dPoint fromPoint = this.grab3dPoint_(new Jun2dPoint(from2dPoint.x(), from2dPoint.y()));
		Jun3dPoint toPoint = this.grab3dPoint_(new Jun2dPoint(to2dPoint.x(), to2dPoint.y()));
		JunAngle rotationAngle = (new Jun3dLine(grabSphere.center(), toPoint)).angleWithLine_(new Jun3dLine(grabSphere.center(), fromPoint));

		if (Math.abs(rotationAngle.rad()) > ThetaAccuracy) {
			Jun3dLine rotationAxis = new Jun3dLine(grabSphere.center(), (Jun3dPoint) grabSphere.center().minus_(((Jun3dPoint) fromPoint.minus_(grabSphere.center())).product_((Jun3dPoint) toPoint.minus_(grabSphere.center()))));
			Jun3dTransformation transformation = Jun3dTransformation.Rotate_around_(rotationAngle, rotationAxis);
			Jun3dTransformation transformationInv = Jun3dTransformation.Rotate_around_(rotationAngle.mul_(-1), rotationAxis);
			Jun3dPoint upPoint = (Jun3dPoint) aProjection.sightPoint().plus_(aProjection.unitUpVector());
			aProjection.eyePoint_(transformationInv.applyTo_(aProjection.eyePoint()));
			aProjection.upVector_((Jun3dPoint) transformation.applyTo_(upPoint).minus_(aProjection.sightPoint()));
			this.openGLProjection_(aProjection);
			this.changed_($("projection"));
		}
	}

	/**
	 * Change the eye point.
	 * 
	 * @param from2dPoint sra.jun.grometry.basic.Jun2dPoint
	 * @param to2dPoint sra.jun.grometry.basic.Jun2dPoint
	 * @category manipulating
	 */
	public void look_xy_(Jun2dPoint from2dPoint, Jun2dPoint to2dPoint) {
		if (this.openGL3dObject() == null) {
			return;
		}

		JunOpenGLProjection aProjection = this.openGLProjection();
		Jun3dPoint fromPoint = this.grab3dPoint_(new Jun2dPoint(from2dPoint.x(), from2dPoint.y()));
		Jun3dPoint toPoint = this.grab3dPoint_(new Jun2dPoint(to2dPoint.x(), to2dPoint.y()));
		aProjection.eyePoint_((Jun3dPoint) aProjection.eyePoint().minus_((Jun3dPoint) toPoint.minus_(fromPoint)));
		this.openGLProjection_(aProjection);
		this.changed_($("projection"));
	}

	/**
	 * Slide the eye beam.
	 * 
	 * @param from2dPoint sra.jun.grometry.basic.Jun2dPoint
	 * @param to2dPoint sra.jun.grometry.basic.Jun2dPoint
	 * @category manipulating
	 */
	public void slide_xy_(Jun2dPoint from2dPoint, Jun2dPoint to2dPoint) {
		if (this.openGL3dObject() == null) {
			return;
		}

		JunOpenGLProjection aProjection = this.openGLProjection();
		Jun3dPoint fromPoint = this.grab3dPoint_(new Jun2dPoint(from2dPoint.x(), from2dPoint.y()));
		Jun3dPoint toPoint = this.grab3dPoint_(new Jun2dPoint(to2dPoint.x(), to2dPoint.y()));
		aProjection.sightPoint_((Jun3dPoint) aProjection.sightPoint().minus_((Jun3dPoint) toPoint.minus_(fromPoint)));
		aProjection.eyePoint_((Jun3dPoint) aProjection.eyePoint().minus_((Jun3dPoint) toPoint.minus_(fromPoint)));
		this.openGLProjection_(aProjection);
		this.changed_($("projection"));
	}

	/**
	 * Add the JunOpenGL3dObject as one of the selected objects.
	 * 
	 * @param a3dObject sra.jun.opengl.objects.Jun3dOpenGLObject
	 * @category selecting
	 */
	public void addSelectedObject_(JunOpenGL3dObject a3dObject) {
		if (this.openGL3dObject() == null) {
			return;
		}

		if (this.openGL3dObject().isPrimitive()) {
			return;
		}

		if (this.selectedObjects().contains(a3dObject) == false) {
			this.selectedObjects().add(a3dObject);
		}
	}

	/**
	 * Clear the variable which holds the currently selected objects.
	 * 
	 * @category selecting
	 */
	public void clearSelectedObjects() {
		selectedObjects = new Vector();
	}

	/**
	 * Remove the JunOpenGL3dObject from the selected objects.
	 * 
	 * @param a3dObject sra.jun.opengl.objects.JunOpenGL3dObject
	 * @category selecting
	 */
	public void removeSelectedObject_(JunOpenGL3dObject a3dObject) {
		if (this.openGL3dObject() == null) {
			return;
		}
		if (this.openGL3dObject().isPrimitive()) {
			return;
		}

		if (this.selectedObjects().contains(a3dObject)) {
			this.selectedObjects().remove(a3dObject);
		}
	}

	/**
	 * Answer my currently selected objects.
	 *
	 * @return java.util.Vector
	 * @category selecting
	 */
	public Vector selectedObjects() {
		if (selectedObjects == null) {
			this.clearSelectedObjects();
		}
		return selectedObjects;
	}

	/**
	 * Dolly the projector.
	 * 
	 * @param factor double
	 * @category projection
	 */
	public void dolly_(double factor) {
		double dollyFactor = factor;

		Jun3dLine aLine = new Jun3dLine(this.eyePoint(), this.sightPoint());
		Jun3dPoint eyePoint = (Jun3dPoint) aLine.atT_(0.0 + dollyFactor);
		Jun3dPoint sightPoint = (Jun3dPoint) aLine.atT_(1.0 + dollyFactor);
		this.openGLProjector.eyePoint_(eyePoint);
		this.openGLProjector.sightPoint_(sightPoint);
		this.changed_($("projection"));
	}

	/**
	 * Answer my current eye point.
	 *
	 * @return sra.jun.geometry.basic.Jun3dPoint
	 * @category projection
	 */
	public Jun3dPoint eyePoint() {
		return this.openGLProjector().eyePoint();
	}

	/**
	 * Set my new eye point.
	 * 
	 * @param a3dPoint sra.jun.grometry.basic.Jun3dPoint
	 * @category projection
	 */
	public void eyePoint_(Jun3dPoint a3dPoint) {
		this.openGLProjector().eyePoint_(a3dPoint);
		this.changed_($("projection"));
	}

	/**
	 * Fit the display object to the view.
	 * 
	 * @category projection
	 */
	public void fit() {
		if (this.openGL3dObject() == null) {
			return;
		}

		JunCursorAnimator cursorAnimator = JunCursorAnimator.ClockCursors();
		try {
			cursorAnimator._show();

			this.fitSilently();
			this.changed_($("projection"));
		} finally {
			cursorAnimator._restore();
		}
	}

	/**
	 * Fit the sight point to the current view.
	 * 
	 * @category projection
	 */
	public void fitSight() {
		if (this.openGL3dObject() == null) {
			return;
		}

		JunCursorAnimator cursorAnimator = JunCursorAnimator.ClockCursors();
		try {
			cursorAnimator._show();

			Jun3dPoint sightPoint = this.computeSightPoint();
			this.sightPoint_(sightPoint);
		} finally {
			cursorAnimator._restore();
		}
	}

	/**
	 * Fit the 3d object to the current view.
	 * 
	 * @category projection
	 */
	public void fitSilently() {
		if (this.openGL3dObject() == null) {
			return;
		}

		JunCursorAnimator cursorAnimator = JunCursorAnimator.ClockCursors();
		try {
			cursorAnimator._show();

			Jun3dPoint sightPoint = this.computeSightPoint();
			this.openGLProjector().sightPoint_(sightPoint);
			double zoomHeight = this.computeZoomHeight();
			this.openGLProjector().zoomHeight_(zoomHeight);
		} finally {
			cursorAnimator._restore();
		}
	}

	/**
	 * Menu message: Fit to the zoom factor.
	 * 
	 * @category projection
	 */
	public void fitZoom() {
		if (this.openGL3dObject() == null) {
			return;
		}

		JunCursorAnimator cursorAnimator = JunCursorAnimator.ClockCursors();
		try {
			cursorAnimator._show();

			double zoomHeight = this.computeZoomHeight();
			this.zoomHeight_(zoomHeight);
		} finally {
			cursorAnimator._restore();
		}
	}

	/**
	 * Pan.
	 * 
	 * @category projection
	 */
	public void pan() {
		this.zoom_(0.5);
	}

	/**
	 * Change the projection to parallel.
	 * 
	 * @category projection
	 */
	public void parallelProjection() {
		this.openGLProjector().parallelProjection();
		this.updateViewMenuIndication();
		this.changed_($("projection"));
	}

	/**
	 * Change the projection to perspective.
	 * 
	 * @category projection
	 */
	public void perspectiveProjection() {
		this.openGLProjector().perspectiveProjection();
		this.updateViewMenuIndication();
		this.changed_($("projection"));
	}

	/**
	 * Answer my current sight point.
	 *
	 * @return sra.jun.geometry.basic.Jun3dPoint
	 * @category projection
	 */
	public Jun3dPoint sightPoint() {
		return this.openGLProjector().sightPoint();
	}

	/**
	 * Set my new sight point.
	 * 
	 * @param a3dPoint sra.jun.grometry.basic.Jun3dPoint
	 * @category projection
	 */
	public void sightPoint_(Jun3dPoint a3dPoint) {
		this.openGLProjector().sightPoint_(a3dPoint);
		this.changed_($("projection"));
	}

	/**
	 * Answer my current up vector.
	 *
	 * @return sra.jun.geometry.basic.Jun3dPoint
	 * @category projection
	 */
	public Jun3dPoint upVector() {
		return this.openGLProjector().upVector();
	}

	/**
	 * Set my new up vector.
	 * 
	 * @param a3dPoint sra.jun.grometry.basic.Jun3dPoint
	 * @category projection
	 */
	public void upVector_(Jun3dPoint a3dPoint) {
		this.openGLProjector().upVector_(a3dPoint);
		this.changed_($("projection"));
	}

	/**
	 * Answer my current view factor.
	 *
	 * @return double
	 * @category projection
	 */
	public double viewFactor() {
		return this.openGLProjector().viewFactor();
	}

	/**
	 * Set my new view factor.
	 * 
	 * @param factor double
	 * @category projection
	 */
	public void viewFactor_(double factor) {
		this.openGLProjector().viewFactor_(factor);
		this.changed_($("projection"));
	}

	/**
	 * Do the zoom.
	 * 
	 * @category projection
	 */
	public void zoom() {
		this.zoom_(2.0);
	}

	/**
	 * Do the zoom with the specified factor.
	 * 
	 * @param factor double
	 * @category projection
	 */
	public void zoom_(double factor) {
		double zoomFactor = factor;

		this.openGLProjector().zoom_(zoomFactor);
		this.changed_($("projection"));
	}

	/**
	 * Answer my current zoom height.
	 *
	 * @return double
	 * @category projection
	 */
	public double zoomHeight() {
		return this.openGLProjector().zoomHeight();
	}

	/**
	 * Set my new zoom height.
	 * 
	 * @param aNumber double
	 * @category projection
	 */
	public void zoomHeight_(double aNumber) {
		this.openGLProjector().zoomHeight_(aNumber);
		this.changed_($("projection"));
	}

	/**
	 * Change the presentation mode to hidden-line.
	 * 
	 * @category presentation
	 */
	public void hiddenlinePresentation() {
		this.openGLProjector().hiddenlinePresentation();
		this.updateViewMenuIndication();
		this.changed_($("presentation"));
	}

	/**
	 * Change the presentation mode to solid.
	 * 
	 * @category presentation
	 */
	public void solidPresentation() {
		this.openGLProjector().solidPresentation();
		this.updateViewMenuIndication();
		this.changed_($("presentation"));
	}

	/**
	 * Change the presentation mode to wireframe.
	 * 
	 * @category presentation
	 */
	public void wireframePresentation() {
		this.openGLProjector().wireframePresentation();
		this.updateViewMenuIndication();
		this.changed_($("presentation"));
	}

	/**
	 * Change the shading mode to flat.
	 * 
	 * @category shading
	 */
	public void flatShading() {
		this.openGLProjector().flatShading();
		this.updateViewMenuIndication();
		this.changed_($("shading"));
	}

	/**
	 * Change the shading mode to smooth.
	 * 
	 * @category shading
	 */
	public void smoothShading() {
		this.openGLProjector().smoothShading();
		this.updateViewMenuIndication();
		this.changed_($("shading"));
	}

	/**
	 * Answer true if the display object has a texture mapping.
	 * 
	 * @return boolean
	 * @category texture mapping
	 */
	public boolean hasTextureMapping() {
		JunOpenGL3dObject displayObject = this.openGL3dObject();
		if (displayObject == null) {
			return false;
		}

		final StValueHolder aBoolean = new StValueHolder(false);

		JunOpenGLTexture aTexture = displayObject.texture();
		if (aTexture != null) {
			aBoolean.value_(true);
		}

		if (displayObject.isCompound()) {
			displayObject.primitivesDo_(new StBlockClosure() {
				public Object value_(Object anObject) {
					JunOpenGL3dObject each = (JunOpenGL3dObject) anObject;
					JunOpenGLTexture anotherTexture = each.texture();
					if (anotherTexture != null) {
						aBoolean.value_(true);
					}
					return null;
				}
			});
		}

		return aBoolean._booleanValue();
	}

	/**
	 * Answer true if the display object need a clamp to be set.
	 * 
	 * @return boolean
	 * @category texture mapping
	 */
	public boolean clamp() {
		JunOpenGL3dObject displayObject = this.openGL3dObject();
		if (displayObject == null) {
			return false;
		}

		final StValueHolder aBoolean = new StValueHolder(false);

		JunOpenGLTexture aTexture = displayObject.texture();
		if (aTexture != null) {
			aBoolean.value_(aTexture.clamp());
		}

		if (displayObject.isCompound()) {
			displayObject.primitivesDo_(new StBlockClosure() {
				public Object value_(Object anObject) {
					JunOpenGL3dObject each = (JunOpenGL3dObject) anObject;
					JunOpenGLTexture anotherTexture = each.texture();
					if (anotherTexture != null) {
						aBoolean.value_(anotherTexture.clamp());
					}
					return null;
				}
			});
		}

		return aBoolean._booleanValue();
	}

	/**
	 * Answer true if the display object need linear to be set.
	 * 
	 * @return boolean
	 */
	public boolean linear() {
		JunOpenGL3dObject displayObject = this.openGL3dObject();
		if (displayObject == null) {
			return false;
		}

		final StValueHolder aBoolean = new StValueHolder(false);

		JunOpenGLTexture aTexture = displayObject.texture();
		if (aTexture != null) {
			aBoolean.value_(aTexture.linear());
		}

		if (displayObject.isCompound()) {
			displayObject.primitivesDo_(new StBlockClosure() {
				public Object value_(Object anObject) {
					JunOpenGL3dObject each = (JunOpenGL3dObject) anObject;
					JunOpenGLTexture texture = each.texture();
					if (texture != null) {
						aBoolean.value_(texture.linear());
					}
					return null;
				}
			});
		}

		return aBoolean._booleanValue();
	}

	/**
	 * Answer true if the display object need mipmap to be set.
	 * 
	 * @return boolean
	 * @category texture mapping
	 */
	public boolean mipmap() {
		JunOpenGL3dObject displayObject = this.openGL3dObject();
		if (displayObject == null) {
			return false;
		}

		final StValueHolder aBoolean = new StValueHolder(false);

		JunOpenGLTexture aTexture = displayObject.texture();
		if (aTexture != null) {
			aBoolean.value_(aTexture.mipmap());
		}

		if (displayObject.isCompound()) {
			displayObject.primitivesDo_(new StBlockClosure() {
				public Object value_(Object anObject) {
					JunOpenGL3dObject each = (JunOpenGL3dObject) anObject;
					JunOpenGLTexture texture = each.texture();
					if (texture != null) {
						aBoolean.value_(texture.mipmap());
					}
					return null;
				}
			});
		}

		return aBoolean._booleanValue();
	}

	/**
	 * Answer true if the display object need modulate to be set.
	 * 
	 * @return boolean
	 * @category texture mapping
	 */
	public boolean modulate() {
		JunOpenGL3dObject displayObject = this.openGL3dObject();
		if (displayObject == null) {
			return false;
		}

		final StValueHolder aBoolean = new StValueHolder(false);

		JunOpenGLTexture aTexture = displayObject.texture();
		if (aTexture != null) {
			aBoolean.value_(aTexture.modulate());
		}

		if (displayObject.isCompound()) {
			displayObject.primitivesDo_(new StBlockClosure() {
				public Object value_(Object anObject) {
					JunOpenGL3dObject each = (JunOpenGL3dObject) anObject;
					JunOpenGLTexture texture = each.texture();
					if (texture != null) {
						aBoolean.value_(texture.modulate());
					}
					return null;
				}
			});
		}

		return aBoolean._booleanValue();
	}

	/**
	 * Answer true if the display object need nearest to be set.
	 * 
	 * @return boolean
	 * @category texture mapping
	 */
	public boolean nearest() {
		return !this.linear();
	}

	/**
	 * Answer true if the display object need repeat to be set.
	 * 
	 * @return boolean
	 * @category texture mapping
	 */
	public boolean repeat() {
		return !this.clamp();
	}

	/**
	 * Answer a default view.
	 * 
	 * @return sra.smalltalk.StView
	 * @category interface opening
	 */
	public StView defaultView() {
		if (GetDefaultViewMode() != VIEW_SWING_ONLY) {
			return new JunOpenGL3dViewAwt(this);
		} else {
			return new JunOpenGL3dViewSwing(this);
		}
	}

	/**
	 * Create a Frame for the application model.
	 * Do everything but open the frame.
	 *
	 * @return java.awt.Frame
	 * @param aView sra.smalltalk.StView
	 * @category interface opening
	 */
	protected Frame allButOpenView_(StView aView) {
		Frame aFrame = super.allButOpenView_(aView);
		// this.flushProjector();
		if (this.defaultProjectionTable().containsKey($("sightPoint")) == false || this.defaultProjectionTable().containsKey($("zoomHeight")) == false) {
			this.fitSilently();
		}
		return aFrame;
	}

	/**
	 * Answer the title string.
	 * 
	 * @return java.lang.String
	 * @category interface opening
	 */
	protected String windowTitle() {
		return "OpenGL 3d Canvas";
	}

	/**
	 * Change the eye point.
	 * 
	 * @category menu messages
	 */
	public void changeEyePoint() {
		Jun3dPoint point = this.eyePoint();
		float x = (float) point.x();
		float y = (float) point.y();
		float z = (float) point.z();
		String string = JunDialog.Request_(JunSystem.$String("Input a new eye point.") + " (x, y, z)", "(" + x + "," + y + "," + z + ")");
		if (string == null || string.length() == 0) {
			return;
		}

		try {
			point = (Jun3dPoint) JunSmallCompiler.Evaluate_(string);
		} catch (Exception e) {
			JunDialog.Warn_(string + JunSystem.$String(" is invalid point."));
		}

		this.eyePoint_(point);
	}

	/**
	 * Menu message: Change the sight point.
	 * 
	 * @category menu messages
	 */
	public void changeSightPoint() {
		Jun3dPoint point = this.sightPoint();
		float x = (float) point.x();
		float y = (float) point.y();
		float z = (float) point.z();
		String string = JunDialog.Request_(JunSystem.$String("Input a new sight point.") + " (x, y, z)", "(" + x + "," + y + "," + z + ")");
		if (string == null || string.length() == 0) {
			return;
		}

		try {
			point = (Jun3dPoint) JunSmallCompiler.Evaluate_(string);
		} catch (Exception e) {
			JunDialog.Warn_(string + JunSystem.$String(" is invalid point."));
		}

		this.sightPoint_(point);
	}

	/**
	 * Menu message: Change the up vector.
	 * 
	 * @category menu messages
	 */
	public void changeUpVector() {
		Jun3dPoint point = this.upVector();
		float x = (float) point.x();
		float y = (float) point.y();
		float z = (float) point.z();
		String string = JunDialog.Request_(JunSystem.$String("Input a new up vector.") + " (x, y, z)", "(" + x + "," + y + "," + z + ")");
		if (string == null || string.length() == 0) {
			return;
		}

		try {
			point = (Jun3dPoint) JunSmallCompiler.Evaluate_(string);
		} catch (Exception e) {
			JunDialog.Warn_(string + JunSystem.$String(" is invalid point."));
		}

		this.upVector_(point);
	}

	/**
	 * Menu message: Change the view factor.
	 * 
	 * @category menu messages
	 */
	public void changeViewFactor() {
		double value = this.viewFactor();
		String string = JunDialog.Request_(JunSystem.$String("Input a new view factor."), String.valueOf((float) value));
		if (string == null) {
			return;
		}

		try {
			value = Double.valueOf(string).doubleValue();
		} catch (Exception e) {
			JunDialog.Warn_(string + JunSystem.$String(" is invalid value."));
		}

		this.viewFactor_(value);
	}

	/**
	 * Menu message: Change the zoom height.
	 * 
	 * @category menu messages
	 */
	public void changeZoomHeight() {
		double value = this.zoomHeight();
		String string = JunDialog.Request_(JunSystem.$String("Input a new zoom height."), String.valueOf((float) value));
		if (string == null) {
			return;
		}

		try {
			value = Double.valueOf(string).doubleValue();
		} catch (Exception e) {
			JunDialog.Warn_(string + JunSystem.$String(" is invalid value."));
		}

		this.zoomHeight_(value);
	}

	/**
	 * Menu message: Close up.
	 * 
	 * @category menu messages
	 */
	public void closeUp() {
		JunOpenGLProjection aProjection = this.openGLProjection();
		double aRadius = aProjection.regularHeight() / 2;
		double aDistance = aProjection.sightPoint().distance_(aProjection.eyePoint());
		Jun3dLine aLine = new Jun3dLine(aProjection.sightPoint(), aProjection.eyePoint());
		Jun3dPoint aPoint = (Jun3dPoint) aLine.atT_(aRadius / aDistance);
		aProjection.eyePoint_(aPoint);
		aProjection.zoomHeight_(this.computeZoomHeight());
		this.openGLProjection_(aProjection);
		this.changed_($("projection"));
	}

	/**
	 * Menu message: Toggle a line smooth effect on a projector. 
	 * 
	 * @category menu messages
	 */
	public void lineSmooth() {
		this.openGLProjector().lineSmooth_(!this.openGLProjector().lineSmooth());
		this.updateViewMenuIndication();
		this.changed_($("smoothing"));
	}

	/**
	 * Menu message: Open the light #1.
	 * 
	 * @category menu messages
	 */
	public void openLight1() {
		JunOpenGLDisplayLight displayLight = this.displayLight1();
		if (displayLight.builder().windows().length > 0) {
			displayLight._windowExpandAndRaise();
		} else {
			displayLight.openPositionLight();
		}
	}

	/**
	 * Menu message: Open the light #2.
	 * 
	 * @category menu messages
	 */
	public void openLight2() {
		JunOpenGLDisplayLight displayLight = this.displayLight2();
		if (displayLight.builder().windows().length > 0) {
			displayLight._windowExpandAndRaise();
		} else {
			displayLight.openPositionLight();
		}
	}

	/**
	 * Menu message: Open the light #3.
	 * 
	 * @category menu messages
	 */
	public void openLight3() {
		JunOpenGLDisplayLight displayLight = this.displayLight3();
		if (displayLight.builder().windows().length > 0) {
			displayLight._windowExpandAndRaise();
		} else {
			displayLight.openPositionLight();
		}
	}

	/**
	 * Menu message: Open the light #4.
	 * 
	 * @category menu messages
	 */
	public void openLight4() {
		JunOpenGLDisplayLight displayLight = this.displayLight4();
		if (displayLight.builder().windows().length > 0) {
			displayLight._windowExpandAndRaise();
		} else {
			displayLight.openPositionLight();
		}
	}

	/**
	 * Menu message: Open the light #5.
	 * 
	 * @category menu messages
	 */
	public void openLight5() {
		JunOpenGLDisplayLight displayLight = this.displayLight5();
		if (displayLight.builder().windows().length > 0) {
			displayLight._windowExpandAndRaise();
		} else {
			displayLight.openAmbientLight();
		}
	}

	/**
	 * Menu message: Toggle a polygon smooth effect on a projector.
	 * 
	 * @category menu messages
	 */
	public void polygonSmooth() {
		this.openGLProjector().polygonSmooth_(!this.openGLProjector().polygonSmooth());
		this.updateViewMenuIndication();
		this.changed_($("smoothing"));
	}

	/**
	 * Menu message: Reset the view.
	 * 
	 * @category menu messages
	 */
	public void resetView() {
		this.flushProjector();
		this.fit();
		this.updateMenuIndication();
	}

	/**
	 * Answer a menu model.
	 * 
	 * @return sra.smalltalk.StMenuModel
	 */
	public StMenuModel menuModel() {
		if (_menuModel == null) {
			_menuModel = this.defaultMenu();
		}
		return _menuModel;
	}

	/**
	 * Answer my light menu.
	 *
	 * @return sra.smalltalk.StMenu
	 */
	protected StMenu _lightMenu() {
		if (_lightMenu == null) {
			_lightMenu = this._defaultLightMenu();
		}
		return _lightMenu;
	}

	/**
	 * Answer my view menu.
	 *
	 * @return sra.smalltalk.StMenu
	 */
	protected StMenu _viewMenu() {
		if (_viewMenu == null) {
			_viewMenu = this._defaultViewMenu();
		}
		return _viewMenu;
	}

	/**
	 * Create a popup menu.
	 * 
	 * @return sra.smalltalk.StMenu
	 */
	protected StMenu popupMenu() {
		StMenu popupMenu = new StMenu("");
		popupMenu.add(this._viewMenu());
		popupMenu.add(this._lightMenu());
		return popupMenu;
	}

	/**
	 * Create the menubar for JunOpenGLDisplay.
	 * 
	 * @return sra.smalltalk.StMenuModel
	 * @category defaults
	 */
	protected StMenuModel defaultMenu() {
		return null;
	}

	/**
	 * Answer the default light menu for JunOpenGL3dModel.
	 * 
	 * @return sra.smalltalk.StMenu
	 */
	protected StMenu _defaultLightMenu() {
		StMenu defaultLightMenu = new StMenu(JunSystem.$String("Light"));

		defaultLightMenu.add(createCheckBoxMenuItem(JunSystem.$String("Light") + "1", "openLight1"));
		defaultLightMenu.add(createCheckBoxMenuItem(JunSystem.$String("Light") + "2", "openLight2"));
		defaultLightMenu.add(createCheckBoxMenuItem(JunSystem.$String("Light") + "3", "openLight3"));
		defaultLightMenu.add(createCheckBoxMenuItem(JunSystem.$String("Light") + "4", "openLight4"));
		defaultLightMenu.addSeparator();
		defaultLightMenu.add(createCheckBoxMenuItem(JunSystem.$String("Ambient Light", "Ambient"), "openLight5"));

		return defaultLightMenu;
	}

	/**
	 * Answer the default view menu for JunOpenGL3dModel.
	 * 
	 * @return sra.smalltalk.StMenu
	 */
	protected StMenu _defaultViewMenu() {
		StMenu defaultViewMenu = new StMenu(JunSystem.$String("View"));

		// Zooming
		StMenu zoomingMenu = new StMenu(JunSystem.$String("Zooming"));
		zoomingMenu.add(createMenuItem(JunSystem.$String("Fit zoom"), "fitZoom"));
		zoomingMenu.add(createMenuItem(JunSystem.$String("Close up"), "closeUp"));
		zoomingMenu.addSeparator();
		zoomingMenu.add(createMenuItem(JunSystem.$String("Zoom"), "zoom"));
		zoomingMenu.add(createMenuItem(JunSystem.$String("Pan"), "pan"));
		defaultViewMenu.add(zoomingMenu);

		// Pointing
		StMenu pointingMenu = new StMenu(JunSystem.$String("Pointing"));
		pointingMenu.add(createMenuItem(JunSystem.$String("Fit sight"), "fitSight"));
		pointingMenu.addSeparator();
		pointingMenu.add(createMenuItem(JunSystem.$String("Eye point") + "...", "changeEyePoint"));
		pointingMenu.add(createMenuItem(JunSystem.$String("Sight point") + "...", "changeSightPoint"));
		pointingMenu.add(createMenuItem(JunSystem.$String("Up vector") + "...", "changeUpVector"));
		pointingMenu.addSeparator();
		pointingMenu.add(createMenuItem(JunSystem.$String("Zoom height") + "...", "changeZoomHeight"));
		pointingMenu.add(createMenuItem(JunSystem.$String("View factor") + "...", "changeViewFactor"));
		defaultViewMenu.add(pointingMenu);

		// Projection
		StMenu projectionMenu = new StMenu(JunSystem.$String("Projection"));
		StButtonGroup projectionGroup = new StButtonGroup();
		projectionMenu.add(createRadioButtonMenuItem(JunSystem.$String("Perspective Projection", "Perspective"), "perspectiveProjection", projectionGroup));
		projectionMenu.add(createRadioButtonMenuItem(JunSystem.$String("Parallel Projection", "Parallel"), "parallelProjection", projectionGroup));
		defaultViewMenu.add(projectionMenu);

		// Presentation
		StMenu presentationMenu = new StMenu(JunSystem.$String("Presentation"));
		StButtonGroup presentationGroup = new StButtonGroup();
		presentationMenu.add(createRadioButtonMenuItem(JunSystem.$String("Solid Presentation", "Solid"), "solidPresentation", presentationGroup));
		presentationMenu.add(createRadioButtonMenuItem(JunSystem.$String("Wireframe Presentation", "Wireframe"), "wireframePresentation", presentationGroup));
		presentationMenu.add(createRadioButtonMenuItem(JunSystem.$String("Hidden-line Presentation", "Hidden-line"), "hiddenlinePresentation", presentationGroup));
		defaultViewMenu.add(presentationMenu);

		// Shading
		StMenu shadingMenu = new StMenu(JunSystem.$String("Shading"));
		StButtonGroup shadingGroup = new StButtonGroup();
		shadingMenu.add(createRadioButtonMenuItem(JunSystem.$String("Flat Shading", "Flat"), "flatShading", shadingGroup));
		shadingMenu.add(createRadioButtonMenuItem(JunSystem.$String("Smooth Shading", "Smooth"), "smoothShading", shadingGroup));
		defaultViewMenu.add(shadingMenu);

		// Smoothing
		StMenu smoothingMenu = new StMenu(JunSystem.$String("Smoothing"));
		smoothingMenu.add(createCheckBoxMenuItem(JunSystem.$String("Line smooth"), "lineSmooth"));
		smoothingMenu.add(createCheckBoxMenuItem(JunSystem.$String("Polygon smooth"), "polygonSmooth"));
		defaultViewMenu.add(smoothingMenu);

		// and others.
		defaultViewMenu.addSeparator();
		defaultViewMenu.add(createMenuItem(JunSystem.$String("Reset"), "resetView"));

		return defaultViewMenu;
	}

	/**
	 * Update the light menu indication.
	 * 
	 * @category menu messages
	 */
	public void updateLightMenuIndication() {
		StMenu lightMenu = this._lightMenu();
		if (lightMenu == null) {
			return;
		}

		String[] lightLabels = { JunSystem.$String("Light") + "1", JunSystem.$String("Light") + "2", JunSystem.$String("Light") + "3", JunSystem.$String("Light") + "4", JunSystem.$String("Ambient Light", "Ambient")};
		for (int i = 0; i < 5; i++) {
			StCheckBoxMenuItem menuItem = (StCheckBoxMenuItem) lightMenu.getItem(lightLabels[i]);
			menuItem.setSelected(this.displayLights()[i].isOn());
		}
	}

	/**
	 * Update the menu indications.
	 * 
	 * @category menu messages
	 */
	public void updateMenuIndication() {
		this.updateViewMenuIndication();
		this.updateLightMenuIndication();
	}

	/**
	 * Update the view menu indication.
	 * 
	 * @category menu messages
	 */
	public void updateViewMenuIndication() {
		StMenu viewMenu = this._viewMenu();
		if (viewMenu == null) {
			return;
		}

		StMenuItem menuItem;
		boolean displayObjectIsNotEmpty = !this.isEmpty();

		// Zooming
		menuItem = viewMenu.getItem("Zooming");
		if (menuItem != null) {
			menuItem.setEnabled(displayObjectIsNotEmpty);
		}

		// Pointing
		menuItem = viewMenu.getItem("Pointing");
		if (menuItem != null) {
			menuItem.setEnabled(displayObjectIsNotEmpty);
		}

		// resetView
		menuItem = viewMenu.getItemWithValue("resetView");
		if (menuItem != null) {
			menuItem.setEnabled(displayObjectIsNotEmpty);
		}

		// Projection
		menuItem = viewMenu.getItem("Projection");
		if (menuItem != null) {
			StMenu projectionMenu = (StMenu) menuItem;
			if (this.openGLProjector().projection() instanceof JunOpenGLPerspective) {
				menuItem = projectionMenu.getItem("Perspective");
				if (menuItem != null) {
					((StRadioButtonMenuItem) menuItem).setSelected(true);
				}
			} else {
				menuItem = projectionMenu.getItem("Parallel");
				if (menuItem != null) {
					((StRadioButtonMenuItem) menuItem).setSelected(true);
				}
			}
		}

		// Presentation
		menuItem = viewMenu.getItem("Presentation");
		if (menuItem != null) {
			StMenu presentationMenu = (StMenu) menuItem;
			if (this.openGLProjector().presentation() == $("solidPresentation")) {
				menuItem = presentationMenu.getItem("Solid");
				if (menuItem != null) {
					((StRadioButtonMenuItem) menuItem).setSelected(true);
				}
			} else if (openGLProjector().presentation() == $("wireframePresentation")) {
				menuItem = presentationMenu.getItem("Wireframe");
				if (menuItem != null) {
					((StRadioButtonMenuItem) menuItem).setSelected(true);
				}
			} else if (openGLProjector().presentation() == $("hiddenlinePresentation")) {
				menuItem = presentationMenu.getItem("Hidden-line");
				if (menuItem != null) {
					((StRadioButtonMenuItem) menuItem).setSelected(true);
				}
			}
		}

		// Shading
		menuItem = viewMenu.getItem("Shading");
		if (menuItem != null) {
			StMenu shadingMenu = (StMenu) menuItem;
			if (this.openGLProjector().shading() == $("flatShading")) {
				menuItem = shadingMenu.getItem("Flat");
				if (menuItem != null) {
					((StRadioButtonMenuItem) menuItem).setSelected(true);
				}
			} else {
				menuItem = shadingMenu.getItem("Smooth");
				if (menuItem != null) {
					((StRadioButtonMenuItem) menuItem).setSelected(true);
				}
			}
		}

		// Smoothing
		menuItem = viewMenu.getItem("Smoothing");
		if (menuItem != null) {
			StMenu smoothingMenu = (StMenu) menuItem;
			menuItem = smoothingMenu.getItem("Line smooth");
			if (menuItem != null) {
				((StCheckBoxMenuItem) menuItem).setSelected(this.openGLProjector().lineSmooth());
			}
			menuItem = smoothingMenu.getItem("Polygon smooth");
			if (menuItem != null) {
				((StCheckBoxMenuItem) menuItem).setSelected(this.openGLProjector().polygonSmooth());
			}
		}
	}

	/**
	 * Answer my current default eye point.
	 * 
	 * @return sra.jun.geometry.basic.Jun3dPoint
	 * @category defaults
	 */
	public Jun3dPoint defaultEyePoint() {
		if (this.openGL3dObject() == null) {
			return new Jun3dPoint(10000, 10000, 10000);
		}

		Jun3dPoint defaultEyePoint = (Jun3dPoint) this.defaultProjectionTable().get($("eyePoint"));
		if (defaultEyePoint != null) {
			return defaultEyePoint;
		} else {
			Jun3dBoundingBox box = this.boundingBox();
			double distance = box.origin().distance_(box.corner());
			distance = distance * 2;
			return new Jun3dPoint(distance, distance, distance);
		}
	}

	/**
	 * Set my new default eye point.
	 * 
	 * @param eyePoint sra.jun.geometry.basic.Jun3dPoint
	 * @category defaults
	 */
	public void defaultEyePoint_(Jun3dPoint eyePoint) {
		this.defaultProjectionTable().put($("eyePoint"), eyePoint);
	}

	/**
	 * Answer the default extent for converting to an image.
	 *
	 * @return java.awt.Dimension
	 * @category defaults
	 */
	public Dimension defaultImageExtent() {
		return DefaultImageExtent;
	}

	/**
	 * Answer the default light color.
	 * 
	 * @return java.awt.Color
	 * @category defaults
	 */
	public Color defaultLightColor() {
		return Color.gray;
	}

	/**
	 * Answer the default light point.
	 * 
	 * @return sra.jun.opengl.objects.Jun3dPoint
	 * @category defaults
	 */
	public Jun3dPoint defaultLightPoint() {
		return new Jun3dPoint(0, 0, 1);
	}

	/**
	 * Answer my current default presentation.
	 *
	 * @return sra.smalltalk.StSymbol
	 * @category defaults
	 */
	public StSymbol defaultPresentation() {
		StSymbol defaultPresentation = (StSymbol) this.defaultProjectionTable().get($("presentation"));
		if (defaultPresentation != null) {
			return defaultPresentation;
		} else {
			return $("solidPresentation");
		}
	}

	/**
	 * Set my new default presentation.
	 *
	 * @param aSymbol sra.smalltalk.StSymbol
	 * @category defaults
	 */
	public void defaultPresentation_(StSymbol aSymbol) {
		if (aSymbol == $("solidPresentation") || aSymbol == $("wireframePresentation") || aSymbol == $("hiddenlinePresentation")) {
			this.defaultProjectionTable().put($("presentation"), aSymbol);
		}
	}

	/**
	 * Answer my current default projection.
	 *
	 * @return sra.smalltalk.StSymbol
	 * @category defaults
	 */
	public StSymbol defaultProjection() {
		StSymbol defaultProjection = (StSymbol) this.defaultProjectionTable().get($("projection"));
		if (defaultProjection != null) {
			return defaultProjection;
		} else {
			return $("perspectiveProjection");
		}
	}

	/**
	 * Set my new default projection.
	 *
	 * @param aSymbol sra.smalltalk.StSymbol
	 * @category defaults
	 */
	public void defaultProjection_(StSymbol aSymbol) {
		if (aSymbol == $("parallelProjection") || aSymbol == $("perspectiveProjection")) {
			this.defaultProjectionTable().put($("projection"), aSymbol);
		}
	}

	/**
	 * Answer my default projection table which holds some default values of the projection.
	 *
	 * @return java.util.Hashtable
	 * @category defaults
	 */
	public Hashtable defaultProjectionTable() {
		if (defaultProjectionTable == null) {
			defaultProjectionTable = new Hashtable(5);
		}
		return defaultProjectionTable;
	}

	/**
	 * Set my new default projection table.
	 *
	 * @param newDefaultProjectionTable java.util.Hashtable
	 * @category defaults
	 */
	public void defaultProjectionTable_(Hashtable newDefaultProjectionTable) {
		defaultProjectionTable = newDefaultProjectionTable;
	}

	/**
	 * Answer my current default sight point.
	 * 
	 * @return sra.jun.geometry.basic.Jun3dPoint
	 * @category defaults
	 */
	public Jun3dPoint defaultSightPoint() {
		if (this.openGL3dObject() == null) {
			return new Jun3dPoint(0, 0, 0);
		}

		Jun3dPoint defaultSightPoint = (Jun3dPoint) this.defaultProjectionTable().get($("sightPoint"));
		if (defaultSightPoint != null) {
			return defaultSightPoint;
		} else {
			return this.boundingBox().center();
		}
	}

	/**
	 * Set my new default sight point.
	 * 
	 * @param sightPoint sra.jun.geometry.basic.Jun3dPoint
	 * @category defaults
	 */
	public void defaultSightPoint_(Jun3dPoint sightPoint) {
		this.defaultProjectionTable().put($("sightPoint"), sightPoint);
	}

	/**
	 * Answer my current default sight point.
	 * 
	 * @return sra.jun.geometry.basic.Jun3dPoint
	 * @category defaults
	 */
	public Jun3dPoint defaultUpVector() {
		Jun3dPoint defaultUpVector = (Jun3dPoint) this.defaultProjectionTable().get($("upVector"));
		if (defaultUpVector != null) {
			return defaultUpVector;
		} else {
			return new Jun3dPoint(-1, -1, 1);
		}
	}

	/**
	 * Set my new default up vector.
	 * 
	 * @param sightPoint sra.jun.geometry.basic.Jun3dPoint
	 * @category defaults
	 */
	public void defaultUpVector_(Jun3dPoint upVector) {
		this.defaultProjectionTable().put($("upVector"), upVector);
	}

	/**
	 * Answer my current default view factor.
	 * 
	 * @return double
	 * @category defaults
	 */
	public double defaultViewFactor() {
		Number defaultViewFactor = (Number) this.defaultProjectionTable().get($("viewFactor"));
		if (defaultViewFactor != null) {
			return defaultViewFactor.doubleValue();
		} else {
			return 10;
		}
	}

	/**
	 * Set my new default view factor.
	 * 
	 * @param viewFactor double
	 * @category defaults
	 */
	public void defaultViewFactor_(double viewFactor) {
		this.defaultProjectionTable().put($("viewFactor"), new Double(viewFactor));
	}

	/**
	 * Answer my current default zoom height.
	 * 
	 * @return double
	 * @category defaults
	 */
	public double defaultZoomHeight() {
		Number defaultZoomHeight = (Number) this.defaultProjectionTable().get($("zoomHeight"));
		if (defaultZoomHeight != null) {
			return defaultZoomHeight.doubleValue();
		} else {
			return this.defaultZoomHeightFactor();
		}
	}

	/**
	 * Set my new default zoom height.
	 * 
	 * @param zoomHeight double
	 * @category defaults
	 */
	public void defaultZoomHeight_(double zoomHeight) {
		this.defaultProjectionTable().put($("zoomHeight"), new Double(zoomHeight));
	}

	/**
	 * Answer the default zoom height factor.
	 *
	 * @return double
	 * @category defaults
	 */
	public double defaultZoomHeightFactor() {
		return 2.4;
	}

	/**
	 * Compute the current sight point.
	 * 
	 * @return sra.jun.geometry.basic.Jun3dPoint
	 * @category private
	 */
	protected Jun3dPoint computeSightPoint() {
		return (Jun3dPoint) this.boundingBox().center();
	}

	/**
	 * Compute the current zoom height.
	 * 
	 * @return double
	 * @category private
	 */
	protected double computeZoomHeight() {
		double depth = this.openGLProjection().distance();
		Jun3dPoint up = this.openGLProjection().translateTo3dPointFromPoint_depth_(new Jun2dPoint(0, -1), depth);
		Jun3dPoint down = this.openGLProjection().translateTo3dPointFromPoint_depth_(new Jun2dPoint(0, 1), depth);
		Jun3dPoint right = this.openGLProjection().translateTo3dPointFromPoint_depth_(new Jun2dPoint(1, 0), depth);
		Jun3dPoint left = this.openGLProjection().translateTo3dPointFromPoint_depth_(new Jun2dPoint(-1, 0), depth);
		final JunPlane horizontal = new JunPlane(this.openGLProjection().eyePoint(), left, right);
		final JunPlane vertical = new JunPlane(this.openGLProjection().eyePoint(), up, down);
		final double maxOffset[] = new double[] { 0.0d, 0.0d };
		this.pointsDo_(new StBlockClosure() {
			public Object value_(Object p) {
				double offsetX = vertical.distanceFromPoint_((Jun3dPoint) p);
				double offsetY = horizontal.distanceFromPoint_((Jun3dPoint) p);
				if (offsetX > maxOffset[0]) {
					maxOffset[0] = offsetX;
				}
				if (offsetY > maxOffset[1]) {
					maxOffset[1] = offsetY;
				}
				return null;
			}
		});
		double maxOffsetX = maxOffset[0];
		double maxOffsetY = maxOffset[1];

		double maxHeight = 0;
		StView aView = this.getView();
		Rectangle aBox = (aView == null) ? null : aView.toComponent().getBounds();
		if (aBox != null && aBox.height != 0 && maxOffsetX != 0 && maxOffsetY != 0) {
			if (maxOffsetY * aBox.width / aBox.height < maxOffsetX) {
				double tmpViewHeight = (aBox.width * maxOffsetY * maxOffsetY) / (aBox.height * maxOffsetX);
				maxHeight = maxOffsetY * maxOffsetY / tmpViewHeight;
			} else {
				maxHeight = maxOffsetY;
			}
		} else {
			maxHeight = Math.max(maxOffsetX, maxOffsetY);
		}

		maxHeight = maxHeight * this.defaultZoomHeightFactor();
		return maxHeight;
	}

	/**
	 * Grab at the specified point.
	 * 
	 * @param aPoint
	 * @return sra.jun.geometry.basic.Jun3dPoint
	 * @category private
	 */
	protected Jun3dPoint grab3dPoint_(Jun2dPoint aPoint) {
		JunOpenGLProjection projection = this.openGLProjection();
		JunSphereSurface handleSphere = this.grabSphere();
		JunOpenGLParallelProjection prj = (JunOpenGLParallelProjection) projection.asParallelProjection();
		Jun3dLine line = new Jun3dLine(prj.translateTo3dPointInNearBoundaryPlaneFromPoint_(aPoint), prj.translateTo3dPointInFarBoundaryPlaneFromPoint_(aPoint));
		Jun3dPoint[] candidates = handleSphere.crossPointsWithLine_(line);

		if (candidates.length == 0) {
			return prj.translateTo3dPointFromPoint_(aPoint);
		} else {
			return candidates[0];
		}
	}

	/**
	 * Create a sphere for grabbing.
	 * 
	 * @return sra.jun.geometry.surfaces.JunSphereSurface
	 * @category private
	 */
	protected JunSphereSurface grabSphere() {
		JunOpenGLProjection projection = this.openGLProjection();
		JunSphereSurface grabSphere = JunSphereSurface.Center_radius_(projection.sightPoint(), projection.regularHeight() / 2);
		return grabSphere;
	}

	/**
	 * Answer a view.
	 * 
	 * @return sra.smalltalk.StView
	 * @see sra.jun.system.framework.JunApplicationModel#getView()
	 * @category private
	 */
	public StView getView() {
		Object[] dependents = this.dependents();
		for (int i = 0; i < dependents.length; i++) {
			Object each = dependents[i];
			if (each instanceof JunOpenGL3dView && ((JunOpenGL3dView) each).getOpenGL3dModel() == this) {
				return (StView) each;
			}
		}

		return null;
	}

}
