package sra.jun.vrml.support;

import java.util.Hashtable;
import sra.jun.vrml.node.abstracts.JunVrmlNode;
import sra.smalltalk.*;

/**
 * JunVrmlSyntaxTreeBuilder class
 * 
 *  @author    ASTI Beijing
 *  @created   UNKNOWN
 *  @updated   2000/03/24 (by nisinaka)
 *  @version   445 (with StPL6.5) based on JunXXX for Smalltalk
 *  @copyright 1999-2004 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2004 Information-technology Promotion Agency, Japan (IPA)
 *  @copyright 2001-2004 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunVrmlSyntaxTreeBuilder.java,v 6.5 2004/04/15 06:28:20 nisinaka Exp $
 */
public abstract class JunVrmlSyntaxTreeBuilder extends StObject {

	/**
	 * Create a new VRML node.
	 * 
	 * @param aSymbol sra.smalltalk.StSymbol
	 * @param anObject java.lang.Object
	 * 
	 * @return sra.jun.vrml.node.abstracts.JunVrmlNode
	 */
	public JunVrmlNode newNode_with_(final StSymbol aSymbol, Object anObject) {
		return this.newNode_with_ifFailed_(aSymbol, anObject, new StBlockClosure() {
			public Object value() {
				throw SmalltalkException.Error("Unknown node - " + aSymbol.toString());
			}
		});
	}

	/**
	 * Create a new VRML node.
	 * 
	 * @param aSymbol sra.smalltalk.StSymbol
	 * @param anObject java.lang.Object
	 * @param failBlock sra.smalltalk.StBlockClosure
	 * 
	 * @return sra.jun.vrml.node.abstracts.JunVrmlNode
	 */
	public JunVrmlNode newNode_with_ifFailed_(StSymbol aSymbol, Object anObject, StBlockClosure failBlock) {
		Class aNodeClass = (Class) this.nodeTypeTable().get(aSymbol);

		if (aNodeClass == null) {
			return (JunVrmlNode) failBlock.value();
		}

		return (JunVrmlNode) _New(aNodeClass, anObject);
	}

	/**
	 * Answer a node type table.
	 * 
	 * @return java.util.Hashtable
	 */
	public abstract Hashtable nodeTypeTable();
}
